//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve RTTE tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_RTTE     1
#define CC_RTTE_DATA_COUNT	18
#define CC_RTTE_SKIP        1   // we take every sample
#define CC_RTTE_ARRAY_SIZE	(CC_RTTE_DATA_COUNT/CC_RTTE_SKIP)
#define CC_RTTE_XUNIT       "C"
#define CC_RTTE_XMIN        -30
#define CC_RTTE_XMAX        150
#define CC_RTTE_XSTEP       10
#define CC_RTTE_YUNIT       "%"
#define CC_RTTE00   1

#define CC_RTTE00_FILE   "CCRTTE00.BIN"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_RTTE_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_RTTE00, "Motor runtime over temperature|de|Motorlaufzeit-Temperatur"/*[TXT_wave_CC_RTTE00]*/ }
	}
}

var_CC_RTTE_Value LIKE VARIABLE var_CC_data_int32{REDEFINE CONSTANT_UNIT CC_RTTE_YUNIT;}
var_CC_RTTE_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_RTTE_YUNIT;}
var_CC_RTTE_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_RTTE_YUNIT;}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC RTTE00
// 
var_CC_RTTE00_Version LIKE VARIABLE var_CC_Version{}
var_CC_RTTE00_Type LIKE VARIABLE var_CC_Type{}
var_CC_RTTE00_Number LIKE VARIABLE var_CC_Number{}
var_CC_RTTE00_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_RTTE00_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_RTTE00_Name LIKE VARIABLE var_CC_Name{}
var_CC_RTTE00_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_RTTE00_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_RTTE00_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_RTTE00_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_RTTE00_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_RTTE_XMIN;}
var_CC_RTTE00_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_RTTE_XSTEP;}
var_CC_RTTE00_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_RTTE00_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_RTTE00_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_RTTE00_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_RTTE00_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_RTTE00_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_RTTE00_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_RTTE_YUNIT;}
var_CC_RTTE00_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_RTTE_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC RTTE00 values
// 
ARRAY arr_CC_RTTE00
{
    LABEL    "CC RTTE00 values";  // [TXT_arr_CC_RTTE00];
	NUMBER_OF_ELEMENTS    CC_RTTE_ARRAY_SIZE;
	TYPE    var_CC_RTTE_Value;
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_RTTE
{
	LABEL 	"Histogram motor runtime over temperature|de|Motorlaufzeit-Temperatur-Histogramm"; // [TXT_graph_CC_RTTE];
	X_AXIS 	xaxis_CC_RTTE;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_RTTE00;
	}
}

AXIS  	xaxis_CC_RTTE
{
    LABEL            "Temperature|de|Temperatur";
    CONSTANT_UNIT    CC_RTTE_XUNIT;
	MIN_VALUE        CC_RTTE_XMIN;
	MAX_VALUE        CC_RTTE_XMAX;
}

AXIS  	yaxis_CC_RTTE
{
    LABEL  "Motor runtime|de|Motorlaufzeit";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_RTTE_MinVal;
	MAX_VALUE    var_CC_RTTE_MaxVal;
}

WAVEFORM  wave_CC_RTTE00
{
	LABEL   "Motor runtime over temperature|de|Motorlaufzeit-Temperatur"; // [TXT_wave_CC_RTTE00];
	Y_AXIS 	yaxis_CC_RTTE;
	TYPE    YT
	{
	    X_INITIAL    var_CC_RTTE00_StartX;
		X_INCREMENT  var_CC_RTTE00_StepX;
		NUMBER_OF_POINTS    CC_RTTE_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_RTTE00
		}
	}
	EMPHASIS  IF (var_CC_RTTE_Selector == CC_RTTE00) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_RTTE main page of CC RTTE
// 
MENU menu_CC_RTTE
{
	LABEL    "Histogram runtime over temperature|de|Motorlaufzeit-Temperatur-Histogramm";  // [TXT_menu_CC_RTTE];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_RTTE00,
		COLUMNBREAK,
	    "\t",  // placeholder
		ROWBREAK,
		graph_CC_RTTE,
		COLUMNBREAK,
		var_CC_RTTE_Selector (NO_LABEL),
		group_CC_RTTE
	}
	PRE_EDIT_ACTIONS
	{
	    calc_CC_RTTE()
	}
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_RTTE  - properties of selected CC RTTE
// 
MENU group_CC_RTTE
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_RTTE_Selector)
		{
			DEFAULT: var_CC_RTTE00_WKN, var_CC_RTTE00_KNR, var_CC_RTTE00_time_created, var_CC_RTTE00_time_stored, var_CC_RTTE00_MinVal, var_CC_RTTE00_MaxVal, var_CC_MotorRuntime, var_CC_RTTE00_FwVersion
			CASE CC_RTTE00: var_CC_RTTE00_WKN, var_CC_RTTE00_KNR, var_CC_RTTE00_time_created, var_CC_RTTE00_time_stored, var_CC_RTTE00_MinVal, var_CC_RTTE00_MaxVal, var_CC_MotorRuntime, var_CC_RTTE00_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_RTTE  - calculate auxiliary values for graph CC RTTE
// 
METHOD calc_CC_RTTE
{
    LABEL    "calc_CC_RTTE"; // [TXT_calc_CC_RTTE];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_RTTE00();

        // MinValue of graph
        var_CC_RTTE_MinVal= 65535;
		if (var_CC_RTTE00_MinVal < var_CC_RTTE_MinVal) var_CC_RTTE_MinVal= var_CC_RTTE00_MinVal;

        // MaxValue of graph
        var_CC_RTTE_MaxVal= -65536;
		if (var_CC_RTTE00_MaxVal > var_CC_RTTE_MaxVal) var_CC_RTTE_MaxVal= var_CC_RTTE00_MaxVal;

		// adjust Min-/MaxVal for better display
		extent= (var_CC_RTTE_MaxVal - var_CC_RTTE_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_RTTE_MaxVal += extent;
		var_CC_RTTE_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_RTTE00  - calculate auxiliary values for CC RTTE00
// 
METHOD calc_CC_RTTE00
{
    LABEL    "calc_CC_RTTE00"; // [TXT_calc_CC_RTTE00];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_RTTE00_MinVal= 65535;
        var_CC_RTTE00_MaxVal= -65536;
		
		for (i= 0; i < CC_RTTE_ARRAY_SIZE; i++)
		{
		    if (arr_CC_RTTE00[i] < var_CC_RTTE00_MinVal) var_CC_RTTE00_MinVal= arr_CC_RTTE00[i];
		    if (arr_CC_RTTE00[i] > var_CC_RTTE00_MaxVal) var_CC_RTTE00_MaxVal= arr_CC_RTTE00[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_RTTE00  - load CC file CCRTTE00.BIN
// 
METHOD load_CC_RTTE00
{
    LABEL    "Load histogram motor runtime temperature|de|Laden Histogram Motorlaufzeit-Temperatur"; // [TXT_load_CC_RTTE00];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_RTTE00
			sz= "Read file " CC_RTTE00_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_RTTE00();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_RTTE();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_RTTE00  -  read and parse CC file
// 
METHOD read_CC_RTTE00
{
    LABEL    "Read file CCRTTE00.BIN|de|Lesen Datei CCRTTE00.BIN"; // [TXT_read_CC_RTTE00];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCRTTE00";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_RTTE00(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_RTTE00_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_RTTE00(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_RTTE00(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
		    // check valid OBEX header for file data
			if ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_RTTE00(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(RTTE00, index);
					    LOAD_CC_TYPE(RTTE00, index);
						LOAD_CC_NUMBER(RTTE00, index);
						LOAD_CC_KNR(RTTE00, index);
						LOAD_CC_WKN(RTTE00, index);
						LOAD_CC_NAME(RTTE00, index);
						LOAD_CC_TIME_STORED(RTTE00, index);
						LOAD_CC_TIME_CREATED(RTTE00, index);
						LOAD_CC_NUMVALUES(RTTE00, index);
						LOAD_CC_BPV(RTTE00, index);
						LOAD_CC_STARTX(RTTE00, index);
						LOAD_CC_STEPX(RTTE00, index);
						LOAD_CC_SETTING(RTTE00, index, 1);
						LOAD_CC_SETTING(RTTE00, index, 2);
						LOAD_CC_SETTING(RTTE00, index, 3);
						LOAD_CC_SETTING(RTTE00, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(RTTE00, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(RTTE00);
						// adjust x-axis properties
						xaxis_CC_RTTE.MIN_VALUE= var_CC_RTTE00_StartX;
						xaxis_CC_RTTE.MAX_VALUE= var_CC_RTTE00_StartX + var_CC_RTTE00_NumValues * CC_RTTE_XSTEP;
						var_CC_RTTE00_StartX += CC_RTTE_XSTEP/2;
						var_CC_RTTE00_StepX= CC_RTTE_XSTEP;
						// adjust motor runtime value
						var_CC_MotorRuntime= var_CC_RTTE00_Setting1 * CC_TO_TIME_VALUE;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_RTTE00_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA1(RTTE00, index, n); 
									CALCULATE_CC_RELVAL(RTTE00, n);
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA2(RTTE00, index, n); 
									CALCULATE_CC_RELVAL(RTTE00, n);
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA4(RTTE00, index, n); 
									CALCULATE_CC_RELVAL(RTTE00, n);
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_RTTE00_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_RTTE00_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_RTTE00(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
