//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve RTTO tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_RTTO     1
#define CC_RTTO_DATA_COUNT	24
#define CC_RTTO_SKIP        1   // we take every sample
#define CC_RTTO_ARRAY_SIZE	(CC_RTTO_DATA_COUNT/CC_RTTO_SKIP)
#define CC_RTTO_XUNIT        "%"
#define CC_RTTO_XMIN        -120
#define CC_RTTO_XMAX        120
#define CC_RTTO_XSTEP       10
#define CC_RTTO_YUNIT        "%"
#define CC_RTTO00   1

#define CC_RTTO00_FILE   "CCRTTO00.BIN"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_RTTO_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_RTTO00, "Motor runtime over torque|de|Motorlaufzeit-Drehmoment" /*[TXT_wave_CC_RTTO00]*/}
	}
}

var_CC_RTTO_Value LIKE VARIABLE var_CC_data_int32{REDEFINE CONSTANT_UNIT CC_RTTO_YUNIT;}
var_CC_RTTO_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_RTTO_YUNIT;}
var_CC_RTTO_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_RTTO_YUNIT;}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC RTTO00
// 
var_CC_RTTO00_Version LIKE VARIABLE var_CC_Version{}
var_CC_RTTO00_Type LIKE VARIABLE var_CC_Type{}
var_CC_RTTO00_Number LIKE VARIABLE var_CC_Number{}
var_CC_RTTO00_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_RTTO00_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_RTTO00_Name LIKE VARIABLE var_CC_Name{}
var_CC_RTTO00_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_RTTO00_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_RTTO00_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_RTTO00_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_RTTO00_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_RTTO_XMIN;}
var_CC_RTTO00_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_RTTO_XSTEP;}
var_CC_RTTO00_Setting1 LIKE VARIABLE var_CC_Setting1{}
var_CC_RTTO00_Setting2 LIKE VARIABLE var_CC_Setting2{}
var_CC_RTTO00_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_RTTO00_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_RTTO00_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_RTTO00_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_RTTO00_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_RTTO_YUNIT;}
var_CC_RTTO00_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_RTTO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC RTTO00 values
// 
ARRAY arr_CC_RTTO00
{
    LABEL    "CC RTTO00 values";  // [TXT_arr_CC_RTTO00];
	NUMBER_OF_ELEMENTS    CC_RTTO_ARRAY_SIZE;
	TYPE    var_CC_RTTO_Value;
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_RTTO
{
	LABEL 	"Histogram motor runtime over torque|de|Motorlaufzeit-Drehmoment-Histogramm"; // [TXT_graph_CC_RTTO];
	X_AXIS 	xaxis_CC_RTTO;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_RTTO00;
	}
}

AXIS  	xaxis_CC_RTTO
{
    LABEL            "Drehmoment";
    CONSTANT_UNIT    CC_RTTO_XUNIT;
	MIN_VALUE        CC_RTTO_XMIN;
	MAX_VALUE        CC_RTTO_XMAX;
}

AXIS  	yaxis_CC_RTTO
{
    LABEL  "Motor runtime|de|Motorlaufzeit";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_RTTO_MinVal;
	MAX_VALUE    var_CC_RTTO_MaxVal;
}

WAVEFORM  wave_CC_RTTO00
{
	LABEL   "Motor runtime over torque|de|Motorlaufzeit-Drehmoment"; // [TXT_wave_CC_RTTO00];
	Y_AXIS 	yaxis_CC_RTTO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_RTTO00_StartX;
		X_INCREMENT  var_CC_RTTO00_StepX;
		NUMBER_OF_POINTS    CC_RTTO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_RTTO00
		}
	}
	EMPHASIS  IF (var_CC_RTTO_Selector == CC_RTTO00) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_RTTO main page of CC RTTO
// 
MENU menu_CC_RTTO
{
	LABEL    "Histogram runtime over torque|de|Motorlaufzeit-Drehmoment-Histogramm";  // [TXT_menu_CC_RTTO];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_RTTO00,
		COLUMNBREAK,
	    "\t",  // placeholder
		ROWBREAK,
		graph_CC_RTTO,
		COLUMNBREAK,
		var_CC_RTTO_Selector (NO_LABEL),
		group_CC_RTTO
	}
	PRE_EDIT_ACTIONS
	{
	    calc_CC_RTTO()
	}
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_RTTO  - properties of selected CC RTTO
// 
MENU group_CC_RTTO
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_RTTO_Selector)
		{
			DEFAULT: var_CC_RTTO00_WKN, var_CC_RTTO00_KNR, var_CC_RTTO00_time_created, var_CC_RTTO00_time_stored, var_CC_RTTO00_MinVal, var_CC_RTTO00_MaxVal, var_CC_MotorRuntime, var_CC_RTTO00_FwVersion
			CASE CC_RTTO00: var_CC_RTTO00_WKN, var_CC_RTTO00_KNR, var_CC_RTTO00_time_created, var_CC_RTTO00_time_stored, var_CC_RTTO00_MinVal, var_CC_RTTO00_MaxVal, var_CC_MotorRuntime, var_CC_RTTO00_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_RTTO  - calculate auxiliary values for graph CC RTTO
// 
METHOD calc_CC_RTTO
{
    LABEL    "calc_CC_RTTO"; // [TXT_calc_CC_RTTO];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_RTTO00();

        // MinValue of graph
        var_CC_RTTO_MinVal= 65535;
		if (var_CC_RTTO00_MinVal < var_CC_RTTO_MinVal) var_CC_RTTO_MinVal= var_CC_RTTO00_MinVal;
        // MaxValue of graph
        var_CC_RTTO_MaxVal= -65536;
		if (var_CC_RTTO00_MaxVal > var_CC_RTTO_MaxVal) var_CC_RTTO_MaxVal= var_CC_RTTO00_MaxVal;

		// adjust Min-/MaxVal for better display
		extent= (var_CC_RTTO_MaxVal - var_CC_RTTO_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_RTTO_MaxVal += extent;
		var_CC_RTTO_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_RTTO00  - calculate auxiliary values for CC RTTO00
// 
METHOD calc_CC_RTTO00
{
    LABEL    "calc_CC_RTTO00"; // [TXT_calc_CC_RTTO00];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_RTTO00_MinVal= 65535;
        var_CC_RTTO00_MaxVal= -65536;
		
		for (i= 0; i < CC_RTTO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_RTTO00[i] < var_CC_RTTO00_MinVal) var_CC_RTTO00_MinVal= arr_CC_RTTO00[i];
		    if (arr_CC_RTTO00[i] > var_CC_RTTO00_MaxVal) var_CC_RTTO00_MaxVal= arr_CC_RTTO00[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_RTTO00  - load CC file CCRTTO00.BIN
// 
METHOD load_CC_RTTO00
{
    LABEL    "Load histogram motor runtime over torque|de|Laden Motorlaufzeit-Drehmoment-Histogram"; // [TXT_load_CC_RTTO00];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_RTTO00
			sz= "Read file " CC_RTTO00_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_RTTO00();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_RTTO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_RTTO00  -  read and parse CC file
// 
METHOD read_CC_RTTO00
{
    LABEL    "Read file CCRTTO00.BIN|de|Lesen Datei CCRTTO00.BIN"; // [TXT_read_CC_RTTO00];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCRTTO00";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_RTTO00(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_RTTO00_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_RTTO00(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_RTTO00(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_RTTO00(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(RTTO00, index);
					    LOAD_CC_TYPE(RTTO00, index);
						LOAD_CC_NUMBER(RTTO00, index);
						LOAD_CC_KNR(RTTO00, index);
						LOAD_CC_WKN(RTTO00, index);
						LOAD_CC_NAME(RTTO00, index);
						LOAD_CC_TIME_STORED(RTTO00, index);
						LOAD_CC_TIME_CREATED(RTTO00, index);
						LOAD_CC_NUMVALUES(RTTO00, index);
						LOAD_CC_BPV(RTTO00, index);
						LOAD_CC_STARTX(RTTO00, index);
						LOAD_CC_STEPX(RTTO00, index);
						LOAD_CC_SETTING(RTTO00, index, 1);
						LOAD_CC_SETTING(RTTO00, index, 2);
						LOAD_CC_SETTING(RTTO00, index, 3);
						LOAD_CC_SETTING(RTTO00, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(RTTO00, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(RTTO00);
						// adjust x-axis properties
						xaxis_CC_RTTO.MIN_VALUE= var_CC_RTTO00_StartX;
						xaxis_CC_RTTO.MAX_VALUE= var_CC_RTTO00_StartX + var_CC_RTTO00_NumValues * CC_RTTO_XSTEP;
						var_CC_RTTO00_StartX += CC_RTTO_XSTEP/2;
						var_CC_RTTO00_StepX= CC_RTTO_XSTEP;
						// adjust motor runtime value
						var_CC_MotorRuntime= var_CC_RTTO00_Setting1 * CC_TO_TIME_VALUE;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_RTTO00_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA1(RTTO00, index, n);
									CALCULATE_CC_RELVAL(RTTO00, n);
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA2(RTTO00, index, n);
									CALCULATE_CC_RELVAL(RTTO00, n);
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
								    LOAD_CC_DATA4(RTTO00, index, n);
									CALCULATE_CC_RELVAL(RTTO00, n);
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_RTTO00_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_RTTO00_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_RTTO00(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
