//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for characteristic curve TOPO tunneled over ServiceChannel 
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define CC_TOPO     1
#define CC_TOPO_DATA_COUNT	1001
#define CC_TOPO_SKIP        1   // we take every sample
#define CC_TOPO_ARRAY_SIZE	(CC_TOPO_DATA_COUNT/CC_TOPO_SKIP)
#define CC_TOPO_XUNIT       "%"
#define CC_TOPO_XMIN        0
#define CC_TOPO_XMAX        100
#define CC_TOPO_XSTEP       0.1
#define CC_TOPO_YUNIT       "%"
#define CC_TOPOC1   1
#define CC_TOPOC2   2
#define CC_TOPOC3   3
#define CC_TOPOO1   4
#define CC_TOPOO2   5
#define CC_TOPOO3   6

#define CC_TOPOC1_FILE   "CCTOPOC1.BIN"
#define CC_TOPOC2_FILE   "CCTOPOC2.BIN"
#define CC_TOPOC3_FILE   "CCTOPOC3.BIN"
#define CC_TOPOO1_FILE   "CCTOPOO1.BIN"
#define CC_TOPOO2_FILE   "CCTOPOO2.BIN"
#define CC_TOPOO3_FILE   "CCTOPOO3.BIN"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VARIABLE var_CC_TOPO_Selector
{
    LABEL    "Curve Selection|de|Kennlinienauswahl";  // [TXT_var_CC_Selector];
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_TOPOC1, "Torque over position, CLOSE (1)|de|Drehmoment-Position, ZU (1)"/*[TXT_wave_CC_TOPOC1]*/ },
        { CC_TOPOC2, "Torque over position, CLOSE (2)|de|Drehmoment-Position, ZU (2)"/*[TXT_wave_CC_TOPOC2]*/ },
        { CC_TOPOC3, "Torque over position, CLOSE (3)|de|Drehmoment-Position, ZU (3)"/*[TXT_wave_CC_TOPOC3]*/ },
        { CC_TOPOO1, "Torque over position, OPEN (1)|de|Drehmoment-Position, AUF (1)"/*[TXT_wave_CC_TOPOO1]*/ },
        { CC_TOPOO2, "Torque over position, OPEN (2)|de|Drehmoment-Position, AUF (2)"/*[TXT_wave_CC_TOPOO2]*/ },
        { CC_TOPOO3, "Torque over position, OPEN (3)|de|Drehmoment-Position, AUF (3)"/*[TXT_wave_CC_TOPOO3]*/ }
	}
	POST_EDIT_ACTIONS
	{
	    DEFINITION { var_CC_TOPO_Direction= var_CC_TOPO_Selector;}
	}
}

var_CC_TOPO_Value LIKE VARIABLE var_CC_data_int8{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPO_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPO_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}

VARIABLE var_CC_TOPO_Direction 
{
    LABEL    [TXT_TXT_150];  // [TXT_TST_596]
	CLASS    LOCAL;
	TYPE     ENUMERATED (2)
	{
        DEFAULT_VALUE	1;
        { CC_TOPOC1, [TXT_TYP_360] },
        { CC_TOPOC2, [TXT_TYP_360] },
        { CC_TOPOC3, [TXT_TYP_360] },
        { CC_TOPOO1, [TXT_TYP_64] },
        { CC_TOPOO2, [TXT_TYP_64] },
        { CC_TOPOO3, [TXT_TYP_64] }
	}
	HANDLING    READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOC1
// 
var_CC_TOPOC1_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOC1_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOC1_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOC1_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOC1_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOC1_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOC1_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOC1_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOC1_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOC1_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOC1_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOC1_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOC1_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_2088.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC1_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2041.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC1_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOC1_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOC1_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOC1_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOC1_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC1_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC1_Toff LIKE VARIABLE PRM_2041 { REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC1_MinToff LIKE VARIABLE PRM_2180 { REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOC1 values
// 
ARRAY arr_CC_TOPOC1
{
    LABEL    "CC TOPOC1 values";  // [TXT_arr_CC_TOPOC1];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_TOPO_Value;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOC2
// 
var_CC_TOPOC2_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOC2_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOC2_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOC2_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOC2_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOC2_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOC2_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOC2_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOC2_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOC2_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOC2_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOC2_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOC2_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_2088.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC2_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2041.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC2_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOC2_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOC2_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOC2_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOC2_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC2_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC2_Toff LIKE VARIABLE PRM_2041 {REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC2_MinToff LIKE VARIABLE PRM_2180 {REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOC2 values
// 
ARRAY arr_CC_TOPOC2
{
    LABEL    "CC TOPOC2 values";  // [TXT_arr_CC_TOPOC2];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_data_int8;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOC3
// 
var_CC_TOPOC3_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOC3_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOC3_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOC3_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOC3_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOC3_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOC3_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOC3_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOC3_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOC3_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOC3_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOC3_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOC3_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_2088.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC3_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2041.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOC3_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOC3_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOC3_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOC3_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOC3_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC3_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC3_Toff LIKE VARIABLE PRM_2041 {REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOC3_MinToff LIKE VARIABLE PRM_2180 {REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOC3 values
// 
ARRAY arr_CC_TOPOC3
{
    LABEL    "CC TOPOC3 values";  // [TXT_arr_CC_TOPOC3];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_TOPO_Value;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOO1
// 
var_CC_TOPOO1_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOO1_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOO1_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOO1_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOO1_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOO1_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOO1_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOO1_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOO1_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOO1_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOO1_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOO1_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOO1_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_1969.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO1_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2036.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO1_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOO1_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOO1_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOO1_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOO1_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO1_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO1_Toff LIKE VARIABLE PRM_2036 {REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO1_MinToff LIKE VARIABLE PRM_2178 {REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOO1 values
// 
ARRAY arr_CC_TOPOO1
{
    LABEL    "CC TOPOO1 values";  // [TXT_arr_CC_TOPOO1];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_TOPO_Value;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOO2
// 
var_CC_TOPOO2_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOO2_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOO2_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOO2_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOO2_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOO2_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOO2_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOO2_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOO2_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOO2_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOO2_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOO2_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOO2_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_1969.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO2_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2036.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO2_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOO2_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOO2_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOO2_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOO2_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO2_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO2_Toff LIKE VARIABLE PRM_2036 {REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT; }
var_CC_TOPOO2_MinToff LIKE VARIABLE PRM_2178 {REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT; }
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOO2 values
// 
ARRAY arr_CC_TOPOO2
{
    LABEL    "CC TOPOO2 values";  // [TXT_arr_CC_TOPOO2];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_data_int8;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLEs for CC TOPOO3
// 
var_CC_TOPOO3_Version LIKE VARIABLE var_CC_Version{}
var_CC_TOPOO3_Type LIKE VARIABLE var_CC_Type{}
var_CC_TOPOO3_Number LIKE VARIABLE var_CC_Number{}
var_CC_TOPOO3_KNR LIKE VARIABLE var_CC_KNR{}
var_CC_TOPOO3_WKN LIKE VARIABLE var_CC_WKN{}
var_CC_TOPOO3_Name LIKE VARIABLE var_CC_Name{}
var_CC_TOPOO3_time_stored LIKE VARIABLE var_CC_time_stored{}
var_CC_TOPOO3_time_created LIKE VARIABLE var_CC_time_created{}
var_CC_TOPOO3_NumValues LIKE VARIABLE var_CC_NumValues{}
var_CC_TOPOO3_BytesPerValue LIKE VARIABLE var_CC_BytesPerValue{}
var_CC_TOPOO3_StartX LIKE VARIABLE var_CC_StartX{REDEFINE DEFAULT_VALUE CC_TOPO_XMIN;}
var_CC_TOPOO3_StepX LIKE VARIABLE var_CC_StepX{REDEFINE DEFAULT_VALUE CC_TOPO_XSTEP;}
var_CC_TOPOO3_Setting1 LIKE VARIABLE var_CC_Setting1
{
    REDEFINE LABEL PRM_1969.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO3_Setting2 LIKE VARIABLE var_CC_Setting2
{
    REDEFINE LABEL PRM_2036.LABEL;
    REDEFINE CONSTANT_UNIT [TXT_TYP_703];  // Nm
}
var_CC_TOPOO3_Setting3 LIKE VARIABLE var_CC_Setting3{}
var_CC_TOPOO3_Setting4 LIKE VARIABLE var_CC_Setting4{}

// CC f/w version only from CC_VERSION >= 3.00
var_CC_TOPOO3_FwVersion LIKE VARIABLE var_CC_FwVersion{REDEFINE VALIDITY	IF ( var_CC_TOPOO3_Version >= CC_VERSION_HDRv3 ) { TRUE; } ELSE { FALSE; }}

var_CC_TOPOO3_MinVal LIKE VARIABLE var_CC_MinVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO3_MaxVal LIKE VARIABLE var_CC_MaxVal{REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO3_Toff LIKE VARIABLE PRM_2036 {REDEFINE CLASS LOCAL; REDEFINE MAX_VALUE 200; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
var_CC_TOPOO3_MinToff LIKE VARIABLE PRM_2178 {REDEFINE CLASS LOCAL; REDEFINE CONSTANT_UNIT CC_TOPO_YUNIT;}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining ARRAY for CC TOPOO3 values
// 
ARRAY arr_CC_TOPOO3
{
    LABEL    "CC TOPOO3 values";  // [TXT_arr_CC_TOPOO3];
	NUMBER_OF_ELEMENTS    CC_TOPO_ARRAY_SIZE;
	TYPE    var_CC_TOPO_Value;
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GRAPHS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GRAPH graph_CC_TOPO
{
	LABEL 	"Characteristic curves torque over position|de|Drehmoment-Position-Kennlinien"; // [TXT_graph_CC_TOPO];
	X_AXIS 	xaxis_CC_TOPO;
	WIDTH	X_SMALL;
	MEMBERS
	{
		WAVEFORM1, wave_CC_TOPO_MINTOFF;
		WAVEFORM2, wave_CC_TOPO_TOFF;
		WAVEFORM3, wave_CC_TOPOC1;
		WAVEFORM4, wave_CC_TOPOC2;
		WAVEFORM5, wave_CC_TOPOC3;
		WAVEFORM6, wave_CC_TOPOO1;
		WAVEFORM7, wave_CC_TOPOO2;
		WAVEFORM8, wave_CC_TOPOO3;
	}
}

AXIS  	xaxis_CC_TOPO
{
    LABEL            "Position|de|Position";
    CONSTANT_UNIT    CC_TOPO_XUNIT;
	MIN_VALUE    CC_TOPO_XMIN;
	MAX_VALUE    CC_TOPO_XMAX;
}

AXIS  	yaxis_CC_TOPO
{
    LABEL  "Torque|de|Drehmoment";
	// no CONSTANT_UNIT -- UNIT given by MIN_/MAX_VALUE
	MIN_VALUE    var_CC_TOPO_MinVal;
	MAX_VALUE    var_CC_TOPO_MaxVal;
}

WAVEFORM  wave_CC_TOPO_MINTOFF
{
	LABEL   SELECT (var_CC_TOPO_Selector) 
	        { 
			    CASE CC_TOPOC1: var_CC_TOPOC1_MinToff.LABEL; 
			    CASE CC_TOPOC2: var_CC_TOPOC2_MinToff.LABEL; 
			    CASE CC_TOPOC3: var_CC_TOPOC3_MinToff.LABEL; 
			    CASE CC_TOPOO1: var_CC_TOPOO1_MinToff.LABEL; 
			    CASE CC_TOPOO2: var_CC_TOPOO2_MinToff.LABEL; 
			    CASE CC_TOPOO3: var_CC_TOPOO3_MinToff.LABEL; 
			}
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    HORIZONTAL
	{
		Y_VALUES     
		{  
		    SELECT (var_CC_TOPO_Selector) 
	        { 
			    CASE CC_TOPOC1: var_CC_TOPOC1_MinToff; 
			    CASE CC_TOPOC2: var_CC_TOPOC2_MinToff; 
			    CASE CC_TOPOC3: var_CC_TOPOC3_MinToff; 
			    CASE CC_TOPOO1: var_CC_TOPOO1_MinToff; 
			    CASE CC_TOPOO2: var_CC_TOPOO2_MinToff; 
			    CASE CC_TOPOO3: var_CC_TOPOO3_MinToff; 
			}
		}
	}
}

WAVEFORM  wave_CC_TOPO_TOFF
{
	LABEL   SELECT (var_CC_TOPO_Selector) 
	        { 
			    CASE CC_TOPOC1: var_CC_TOPOC1_Toff.LABEL; 
			    CASE CC_TOPOC2: var_CC_TOPOC2_Toff.LABEL; 
			    CASE CC_TOPOC3: var_CC_TOPOC3_Toff.LABEL; 
			    CASE CC_TOPOO1: var_CC_TOPOO1_Toff.LABEL; 
			    CASE CC_TOPOO2: var_CC_TOPOO2_Toff.LABEL; 
			    CASE CC_TOPOO3: var_CC_TOPOO3_Toff.LABEL; 
			}
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    HORIZONTAL
	{
		Y_VALUES     
		{  
		    SELECT (var_CC_TOPO_Selector) 
	        { 
			    CASE CC_TOPOC1: var_CC_TOPOC1_Toff; 
			    CASE CC_TOPOC2: var_CC_TOPOC2_Toff; 
			    CASE CC_TOPOC3: var_CC_TOPOC3_Toff; 
			    CASE CC_TOPOO1: var_CC_TOPOO1_Toff; 
			    CASE CC_TOPOO2: var_CC_TOPOO2_Toff; 
			    CASE CC_TOPOO3: var_CC_TOPOO3_Toff; 
			}
		}
	}
}

WAVEFORM  wave_CC_TOPOC1
{
	LABEL   "Torque over position, CLOSE (1)|de|Drehmoment-Position, ZU (1)"; // [TXT_wave_CC_TOPOC1];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOC1_StartX;
		X_INCREMENT  var_CC_TOPOC1_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOC1
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOC1) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TOPOC2
{
	LABEL   "Torque over position, CLOSE (2)|de|Drehmoment-Position, ZU (2)"; // [TXT_wave_CC_TOPOC2];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOC2_StartX;
		X_INCREMENT  var_CC_TOPOC2_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOC2
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOC2) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TOPOC3
{
	LABEL   "Torque over position, CLOSE (3)|de|Drehmoment-Position, ZU (3)"; // [TXT_wave_CC_TOPOC3];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOC3_StartX;
		X_INCREMENT  var_CC_TOPOC3_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOC3
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOC3) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TOPOO1
{
	LABEL   "Torque over position, OPEN (1)|de|Drehmoment-Position, AUF (1)"; // [TXT_wave_CC_TOPOO1];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOO1_StartX;
		X_INCREMENT  var_CC_TOPOO1_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOO1
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOO1) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TOPOO2
{
	LABEL   "Torque over position, OPEN (2)|de|Drehmoment-Position, AUF (2)"; // [TXT_wave_CC_TOPOO2];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOO2_StartX;
		X_INCREMENT  var_CC_TOPOO2_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOO2
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOO2) {TRUE;} ELSE {FALSE;}
}

WAVEFORM  wave_CC_TOPOO3
{
	LABEL   "Torque over position, OPEN (3)|de|Drehmoment-Position, AUF (3)"; // [TXT_wave_CC_TOPOO3];
	Y_AXIS 	yaxis_CC_TOPO;
	TYPE    YT
	{
	    X_INITIAL    var_CC_TOPOO3_StartX;
		X_INCREMENT  var_CC_TOPOO3_StepX;
		NUMBER_OF_POINTS    CC_TOPO_ARRAY_SIZE;
		Y_VALUES     
		{  
		    arr_CC_TOPOO3
		}
	}
	EMPHASIS  IF (var_CC_TOPO_Selector == CC_TOPOO3) {TRUE;} ELSE {FALSE;}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU menu_CC_TOPO main page of CC TOPO
// 
MENU menu_CC_TOPO
{
	LABEL    "Curve torque over position|de|Drehmoment-Position-Kennlinie";  // [TXT_menu_CC_TOPO];
	STYLE    WINDOW;
	ACCESS   OFFLINE;
	ITEMS
	{
	    load_CC_TOPOC1,
		COLUMNBREAK,
	    load_CC_TOPOC2,
		COLUMNBREAK,
	    load_CC_TOPOC3,
		ROWBREAK,
	    load_CC_TOPOO1,
		COLUMNBREAK,
	    load_CC_TOPOO2,
		COLUMNBREAK,
	    load_CC_TOPOO3,
		ROWBREAK,
		graph_CC_TOPO,
		COLUMNBREAK,
		var_CC_TOPO_Selector (NO_LABEL),
		group_CC_TOPO
	}
	PRE_EDIT_ACTIONS
	{
		calc_CC_TOPO()
	}
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining MENU group_CC_TOPO  - properties of selected CC TOPO
// 
MENU group_CC_TOPO
{
	LABEL    "";
	STYLE    GROUP;
	ACCESS   OFFLINE;
	ITEMS
	{
	    SELECT (var_CC_TOPO_Selector)
		{
			DEFAULT: var_CC_TOPOC1_WKN, var_CC_TOPOC1_KNR, var_CC_TOPOC1_time_created, var_CC_TOPOC1_time_stored, var_CC_TOPOC1_MinVal, var_CC_TOPOC1_MaxVal, var_CC_TOPOC1_FwVersion
			CASE CC_TOPOC1: var_CC_TOPOC1_WKN, var_CC_TOPOC1_KNR, var_CC_TOPOC1_time_created, var_CC_TOPOC1_time_stored, var_CC_TOPOC1_MinVal, var_CC_TOPOC1_MaxVal, var_CC_TOPOC1_Setting1,  var_CC_TOPOC1_Toff, var_CC_TOPOC1_MinToff, var_CC_TOPO_Direction, var_CC_TOPOC1_FwVersion
			CASE CC_TOPOC2: var_CC_TOPOC2_WKN, var_CC_TOPOC2_KNR, var_CC_TOPOC2_time_created, var_CC_TOPOC2_time_stored, var_CC_TOPOC1_MinVal, var_CC_TOPOC2_MaxVal, var_CC_TOPOC2_Setting1,  var_CC_TOPOC2_Toff, var_CC_TOPOC2_MinToff, var_CC_TOPO_Direction, var_CC_TOPOC2_FwVersion
			CASE CC_TOPOC3: var_CC_TOPOC3_WKN, var_CC_TOPOC3_KNR, var_CC_TOPOC3_time_created, var_CC_TOPOC3_time_stored, var_CC_TOPOC1_MinVal, var_CC_TOPOC3_MaxVal, var_CC_TOPOC3_Setting1,  var_CC_TOPOC3_Toff, var_CC_TOPOC3_MinToff, var_CC_TOPO_Direction, var_CC_TOPOC3_FwVersion
			CASE CC_TOPOO1: var_CC_TOPOO1_WKN, var_CC_TOPOO1_KNR, var_CC_TOPOO1_time_created, var_CC_TOPOO1_time_stored, var_CC_TOPOO1_MinVal, var_CC_TOPOO1_MaxVal, var_CC_TOPOO1_Setting1,  var_CC_TOPOO1_Toff, var_CC_TOPOO1_MinToff, var_CC_TOPO_Direction, var_CC_TOPOO1_FwVersion
			CASE CC_TOPOO2: var_CC_TOPOO2_WKN, var_CC_TOPOO2_KNR, var_CC_TOPOO2_time_created, var_CC_TOPOO2_time_stored, var_CC_TOPOO2_MinVal, var_CC_TOPOO2_MaxVal, var_CC_TOPOO2_Setting1,  var_CC_TOPOO2_Toff, var_CC_TOPOO2_MinToff, var_CC_TOPO_Direction, var_CC_TOPOO2_FwVersion
			CASE CC_TOPOO3: var_CC_TOPOO3_WKN, var_CC_TOPOO3_KNR, var_CC_TOPOO3_time_created, var_CC_TOPOO3_time_stored, var_CC_TOPOO3_MinVal, var_CC_TOPOO3_MaxVal, var_CC_TOPOO3_Setting1,  var_CC_TOPOO3_Toff, var_CC_TOPOO3_MinToff, var_CC_TOPO_Direction, var_CC_TOPOO3_FwVersion
		}
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPO  - calculate auxiliary values for graph CC TOPO
// 
METHOD calc_CC_TOPO
{
    LABEL    "calc_CC_TOPO"; // [TXT_calc_CC_TOPO];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    int extent;
	    // calculate properties of waveforms
	    calc_CC_TOPOC1();
	    calc_CC_TOPOC2();
	    calc_CC_TOPOC3();
	    calc_CC_TOPOO1();
	    calc_CC_TOPOO2();
	    calc_CC_TOPOO3();

        // MinValue of graph (including Toff values)
        var_CC_TOPO_MinVal= var_CC_TOPOC1_MinToff;
		if (var_CC_TOPOO1_MinToff < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOO1_MinToff;
		if (var_CC_TOPOC1_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOC1_MinVal;
		if (var_CC_TOPOC2_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOC2_MinVal;
		if (var_CC_TOPOC3_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOC3_MinVal;
		if (var_CC_TOPOO1_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOC1_MinVal;
		if (var_CC_TOPOO2_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOO2_MinVal;
		if (var_CC_TOPOO3_MinVal < var_CC_TOPO_MinVal) var_CC_TOPO_MinVal= var_CC_TOPOO3_MinVal;

        // MaxValue of graph (including Toff values)
        var_CC_TOPO_MaxVal= var_CC_TOPOC1_Toff;
		if (var_CC_TOPOO1_Toff > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOO1_Toff;
		if (var_CC_TOPOC1_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOC1_MaxVal;
		if (var_CC_TOPOC2_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOC2_MaxVal;
		if (var_CC_TOPOC3_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOC3_MaxVal;
		if (var_CC_TOPOO1_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOC1_MaxVal;
		if (var_CC_TOPOO2_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOO2_MaxVal;
		if (var_CC_TOPOO3_MaxVal > var_CC_TOPO_MaxVal) var_CC_TOPO_MaxVal= var_CC_TOPOO3_MaxVal;

		// adjust Min-/MaxVal for better display
		extent= (var_CC_TOPO_MaxVal - var_CC_TOPO_MinVal)/CC_RANGE_EXTENT_DIVIDER;
		var_CC_TOPO_MaxVal += extent;
		var_CC_TOPO_MinVal -= extent;
		
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOC1  - calculate auxiliary values for CC TOPOC1
// 
METHOD calc_CC_TOPOC1
{
    LABEL    "calc_CC_TOPOC1"; // [TXT_calc_CC_TOPOC1];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOC1_Setting1 != 0)
		{
		    var_CC_TOPOC1_MinToff= (100 * var_CC_TOPOC1_Setting3)/var_CC_TOPOC1_Setting1;
		    var_CC_TOPOC1_Toff= (100 * var_CC_TOPOC1_Setting2)/var_CC_TOPOC1_Setting1;
		}
		else
		{
		    var_CC_TOPOC1_MinToff= 0;
		    var_CC_TOPOC1_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOC1_MinVal= 65535;
        var_CC_TOPOC1_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOC1[i] < var_CC_TOPOC1_MinVal) var_CC_TOPOC1_MinVal= arr_CC_TOPOC1[i];
		    if (arr_CC_TOPOC1[i] > var_CC_TOPOC1_MaxVal) var_CC_TOPOC1_MaxVal= arr_CC_TOPOC1[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOC2  - calculate auxiliary values for CC TOPOC2
// 
METHOD calc_CC_TOPOC2
{
    LABEL    "calc_CC_TOPOC2"; // [TXT_calc_CC_TOPOC2];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOC2_Setting1 != 0)
		{
		    var_CC_TOPOC2_MinToff= (100 * var_CC_TOPOC2_Setting3)/var_CC_TOPOC2_Setting1;
		    var_CC_TOPOC2_Toff= (100 * var_CC_TOPOC2_Setting2)/var_CC_TOPOC2_Setting1;
		}
		else
		{
		    var_CC_TOPOC2_MinToff= 0;
		    var_CC_TOPOC2_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOC2_MinVal= 65535;
        var_CC_TOPOC2_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOC2[i] < var_CC_TOPOC2_MinVal) var_CC_TOPOC2_MinVal= arr_CC_TOPOC2[i];
		    if (arr_CC_TOPOC2[i] > var_CC_TOPOC2_MaxVal) var_CC_TOPOC2_MaxVal= arr_CC_TOPOC2[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOC3  - calculate auxiliary values for CC TOPOC3
// 
METHOD calc_CC_TOPOC3
{
    LABEL    "calc_CC_TOPOC3"; // [TXT_calc_CC_TOPOC3];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOC3_Setting1 != 0)
		{
		    var_CC_TOPOC3_MinToff= (100 * var_CC_TOPOC3_Setting3)/var_CC_TOPOC3_Setting1;
		    var_CC_TOPOC3_Toff= (100 * var_CC_TOPOC3_Setting2)/var_CC_TOPOC3_Setting1;
		}
		else
		{
		    var_CC_TOPOC3_MinToff= 0;
		    var_CC_TOPOC3_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOC3_MinVal= 65535;
        var_CC_TOPOC3_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOC3[i] < var_CC_TOPOC3_MinVal) var_CC_TOPOC3_MinVal= arr_CC_TOPOC3[i];
		    if (arr_CC_TOPOC3[i] > var_CC_TOPOC3_MaxVal) var_CC_TOPOC3_MaxVal= arr_CC_TOPOC3[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOO1  - calculate auxiliary values for CC TOPOO1
// 
METHOD calc_CC_TOPOO1
{
    LABEL    "calc_CC_TOPOO1"; // [TXT_calc_CC_TOPOO1];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOO1_Setting1 != 0)
		{
		    var_CC_TOPOO1_MinToff= (100 * var_CC_TOPOO1_Setting3)/var_CC_TOPOO1_Setting1;
		    var_CC_TOPOO1_Toff= (100 * var_CC_TOPOO1_Setting2)/var_CC_TOPOO1_Setting1;
		}
		else
		{
		    var_CC_TOPOO1_MinToff= 0;
		    var_CC_TOPOO1_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOO1_MinVal= 65535;
        var_CC_TOPOO1_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOO1[i] < var_CC_TOPOO1_MinVal) var_CC_TOPOO1_MinVal= arr_CC_TOPOO1[i];
		    if (arr_CC_TOPOO1[i] > var_CC_TOPOO1_MaxVal) var_CC_TOPOO1_MaxVal= arr_CC_TOPOO1[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOO2  - calculate auxiliary values for CC TOPOO2
// 
METHOD calc_CC_TOPOO2
{
    LABEL    "calc_CC_TOPOO2"; // [TXT_calc_CC_TOPOO2];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOO2_Setting1 != 0)
		{
		    var_CC_TOPOO2_MinToff= (100 * var_CC_TOPOO2_Setting3)/var_CC_TOPOO2_Setting1;
		    var_CC_TOPOO2_Toff= (100 * var_CC_TOPOO2_Setting2)/var_CC_TOPOO2_Setting1;
		}
		else
		{
		    var_CC_TOPOO2_MinToff= 0;
		    var_CC_TOPOO2_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOO2_MinVal= 65535;
        var_CC_TOPOO2_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOO2[i] < var_CC_TOPOO2_MinVal) var_CC_TOPOO2_MinVal= arr_CC_TOPOO2[i];
		    if (arr_CC_TOPOO2[i] > var_CC_TOPOO2_MaxVal) var_CC_TOPOO2_MaxVal= arr_CC_TOPOO2[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining calc_CC_TOPOO3  - calculate auxiliary values for CC TOPOO3
// 
METHOD calc_CC_TOPOO3
{
    LABEL    "calc_CC_TOPOO3"; // [TXT_calc_CC_TOPOO3];
	ACCESS    OFFLINE;
    DEFINITION
    {
	    unsigned int i;
		// calculate related Toff values
		if (var_CC_TOPOO3_Setting1 != 0)
		{
		    var_CC_TOPOO3_MinToff= (100 * var_CC_TOPOO3_Setting3)/var_CC_TOPOO3_Setting1;
		    var_CC_TOPOO3_Toff= (100 * var_CC_TOPOO3_Setting2)/var_CC_TOPOO3_Setting1;
		}
		else
		{
		    var_CC_TOPOO3_MinToff= 0;
		    var_CC_TOPOO3_Toff= 0;
		}

		// adjust y-axis properties
        // Min/MaxValue
        var_CC_TOPOO3_MinVal= 65535;
        var_CC_TOPOO3_MaxVal= -65536;
		
		for (i= 0; i < CC_TOPO_ARRAY_SIZE; i++)
		{
		    if (arr_CC_TOPOO3[i] < var_CC_TOPOO3_MinVal) var_CC_TOPOO3_MinVal= arr_CC_TOPOO3[i];
		    if (arr_CC_TOPOO3[i] > var_CC_TOPOO3_MaxVal) var_CC_TOPOO3_MaxVal= arr_CC_TOPOO3[i];
		}
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOC1  - load CC file CCTOPOC1.BIN
// 
METHOD load_CC_TOPOC1
{
    LABEL    "Load chart torque over position, CLOSE (1)|de|Laden Drehmoment-Position-Kennlinie, ZU (1)"; // [TXT_load_CC_TOPOC1];
	ACCESS    OFFLINE;
	TYPE     unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOC1
			sz= "Read file " CC_TOPOC1_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOC1();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOC2  - load CC file CCTOPOC2.BIN
// 
METHOD load_CC_TOPOC2
{
    LABEL    "Load chart torque over position, CLOSE (2)|de|Laden Drehmoment-Position-Kennlinie, ZU (2)"; // [TXT_load_CC_TOPOC2];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOC2
			sz= "Read file " CC_TOPOC2_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOC2();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOC3  - load CC file CCTOPOC3.BIN
// 
METHOD load_CC_TOPOC3
{
    LABEL    "Load chart torque over position, CLOSE (3)|de|Laden Drehmoment-Position-Kennlinie, ZU (3)"; // [TXT_load_CC_TOPOC3];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOC3
			sz= "Read file " CC_TOPOC3_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOC3();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOO1  - load CC file CCTOPOO1.BIN
// 
METHOD load_CC_TOPOO1
{
    LABEL    "Load chart torque over position, OPEN (1)|de|Laden Drehmoment-Position-Kennlinie, AUF (1)"; // [TXT_load_CC_TOPOO1];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOO1
			sz= "Read file " CC_TOPOO1_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOO1();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOO2  - load CC file CCTOPOO2.BIN
// 
METHOD load_CC_TOPOO2
{
    LABEL    "Load chart torque over position, OPEN (2)|de|Laden Drehmoment-Position-Kennlinie, AUF (2)"; // [TXT_load_CC_TOPOO2];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOO2
			sz= "Read file " CC_TOPOO2_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOO2();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining load_CC_TOPOO3  - load CC file CCTOPOO3.BIN
// 
METHOD load_CC_TOPOO3
{
    LABEL    "Load chart torque over position, OPEN (3)|de|Laden Drehmoment-Position-Kennlinie, AUF (3)"; // [TXT_load_CC_TOPOO3];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
	    unsigned int rc;
		char  sz[];
		
		// establish connection to device
		// show status
		sz= "Connect to device..."; 
		PUT_MESSAGE (sz);
		rc= connect_FT_LOGFILES();
		if (SC_OKAY == rc)
		{
		    // ----------------------------------
			// read CC_TOPOO3
			sz= "Read file " CC_TOPOO3_FILE "..."; 
			PUT_MESSAGE (sz);
			rc= read_CC_TOPOO3();
			switch (rc)
			{
			    default:
				case SC_OKAY:  
				    calc_CC_TOPO();
				    sz += " finished (" + itoa(var_CC_DataIndex) + " data items)"; 
				    break;
				case SC_ERROR: 
				    sz += " failed!"; 
					break;
				case SC_NOT_FOUND:  
				    sz += " not found!"; 
					break;
			}
			PUT_MESSAGE (sz);			

		    ////// FT_DISCONNECT: End connection
		    sz= "Disconnect from device..."; 
		    PUT_MESSAGE (sz);
			rc= disconnect_FT();
		}
		if (SC_ERROR == rc)
		{
		    sz += " failed!";
		}
		else
		{
		    sz= "Done!"; 
		}
		PUT_MESSAGE (sz);
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOC1  -  read and parse CC file
// 
METHOD read_CC_TOPOC1
{
    LABEL    "Read file CCTOPOC1.BIN|de|Lesen Datei CCTOPOC1.BIN"; // [TXT_read_CC_TOPOC1];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOC1";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOC1(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOC1_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOC1(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOC1(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOC1(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOC1, index);
					    LOAD_CC_TYPE(TOPOC1, index);
						LOAD_CC_NUMBER(TOPOC1, index);
						LOAD_CC_KNR(TOPOC1, index);
						LOAD_CC_WKN(TOPOC1, index);
						LOAD_CC_NAME(TOPOC1, index);
						LOAD_CC_TIME_STORED(TOPOC1, index);
						LOAD_CC_TIME_CREATED(TOPOC1, index);
						LOAD_CC_NUMVALUES(TOPOC1, index);
						LOAD_CC_BPV(TOPOC1, index);
						LOAD_CC_STARTX(TOPOC1, index);
						LOAD_CC_STEPX(TOPOC1, index);
						LOAD_CC_SETTING(TOPOC1, index, 1);
						LOAD_CC_SETTING(TOPOC1, index, 2);
						LOAD_CC_SETTING(TOPOC1, index, 3);
						LOAD_CC_SETTING(TOPOC1, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOC1, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOC1);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOC1_StartX= CC_TOPO_XMIN;
						var_CC_TOPOC1_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOC1_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOC1, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOC1, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOC1, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOC1_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOC1_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOC1(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOC2  -  read and parse CC file
// 
METHOD read_CC_TOPOC2
{
    LABEL    "Read file CCTOPOC2.BIN|de|Lesen Datei CCTOPOC2.BIN"; // [TXT_read_CC_TOPOC2];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOC2";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOC2(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOC2_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOC2(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOC2(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOC2(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOC2, index);
					    LOAD_CC_TYPE(TOPOC2, index);
						LOAD_CC_NUMBER(TOPOC2, index);
						LOAD_CC_KNR(TOPOC2, index);
						LOAD_CC_WKN(TOPOC2, index);
						LOAD_CC_NAME(TOPOC2, index);
						LOAD_CC_TIME_STORED(TOPOC2, index);
						LOAD_CC_TIME_CREATED(TOPOC2, index);
						LOAD_CC_NUMVALUES(TOPOC2, index);
						LOAD_CC_BPV(TOPOC2, index);
						LOAD_CC_STARTX(TOPOC2, index);
						LOAD_CC_STEPX(TOPOC2, index);
						LOAD_CC_SETTING(TOPOC2, index, 1);
						LOAD_CC_SETTING(TOPOC2, index, 2);
						LOAD_CC_SETTING(TOPOC2, index, 3);
						LOAD_CC_SETTING(TOPOC2, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOC2, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOC2);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOC2_StartX= CC_TOPO_XMIN;
						var_CC_TOPOC2_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOC2_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOC2, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOC2, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOC2, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOC2_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOC2_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOC2(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOC3  -  read and parse CC file
// 
METHOD read_CC_TOPOC3
{
    LABEL    "Read file CCTOPOC3.BIN|de|Lesen Datei CCTOPOC3.BIN"; // [TXT_read_CC_TOPOC3];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOC3";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOC3(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOC3_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOC3(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOC3(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOC3(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOC3, index);
					    LOAD_CC_TYPE(TOPOC3, index);
						LOAD_CC_NUMBER(TOPOC3, index);
						LOAD_CC_KNR(TOPOC3, index);
						LOAD_CC_WKN(TOPOC3, index);
						LOAD_CC_NAME(TOPOC3, index);
						LOAD_CC_TIME_STORED(TOPOC3, index);
						LOAD_CC_TIME_CREATED(TOPOC3, index);
						LOAD_CC_NUMVALUES(TOPOC3, index);
						LOAD_CC_BPV(TOPOC3, index);
						LOAD_CC_STARTX(TOPOC3, index);
						LOAD_CC_STEPX(TOPOC3, index);
						LOAD_CC_SETTING(TOPOC3, index, 1);
						LOAD_CC_SETTING(TOPOC3, index, 2);
						LOAD_CC_SETTING(TOPOC3, index, 3);
						LOAD_CC_SETTING(TOPOC3, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOC3, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOC3);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOC3_StartX= CC_TOPO_XMIN;
						var_CC_TOPOC3_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOC3_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOC3, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOC3, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOC3, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOC3_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOC3_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOC3(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOO1  -  read and parse CC file
// 
METHOD read_CC_TOPOO1
{
    LABEL    "Read file CCTOPOO1.BIN|de|Lesen Datei CCTOPOO1.BIN"; // [TXT_read_CC_TOPOO1];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOO1";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOO1(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOO1_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOO1(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOO1(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOO1(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOO1, index);
					    LOAD_CC_TYPE(TOPOO1, index);
						LOAD_CC_NUMBER(TOPOO1, index);
						LOAD_CC_KNR(TOPOO1, index);
						LOAD_CC_WKN(TOPOO1, index);
						LOAD_CC_NAME(TOPOO1, index);
						LOAD_CC_TIME_STORED(TOPOO1, index);
						LOAD_CC_TIME_CREATED(TOPOO1, index);
						LOAD_CC_NUMVALUES(TOPOO1, index);
						LOAD_CC_BPV(TOPOO1, index);
						LOAD_CC_STARTX(TOPOO1, index);
						LOAD_CC_STEPX(TOPOO1, index);
						LOAD_CC_SETTING(TOPOO1, index, 1);
						LOAD_CC_SETTING(TOPOO1, index, 2);
						LOAD_CC_SETTING(TOPOO1, index, 3);
						LOAD_CC_SETTING(TOPOO1, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOO1, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOO1);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOO1_StartX= CC_TOPO_XMIN;
						var_CC_TOPOO1_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOO1_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOO1, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOO1, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOO1, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOO1_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOO1_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOO1(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOO2  -  read and parse CC file
// 
METHOD read_CC_TOPOO2
{
    LABEL    "Read file CCTOPOO2.BIN|de|Lesen Datei CCTOPOO2.BIN"; // [TXT_read_CC_TOPOO2];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOO2";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOO2(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOO2_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOO2(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOO2(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOO2(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOO2, index);
					    LOAD_CC_TYPE(TOPOO2, index);
						LOAD_CC_NUMBER(TOPOO2, index);
						LOAD_CC_KNR(TOPOO2, index);
						LOAD_CC_WKN(TOPOO2, index);
						LOAD_CC_NAME(TOPOO2, index);
						LOAD_CC_TIME_STORED(TOPOO2, index);
						LOAD_CC_TIME_CREATED(TOPOO2, index);
						LOAD_CC_NUMVALUES(TOPOO2, index);
						LOAD_CC_BPV(TOPOO2, index);
						LOAD_CC_STARTX(TOPOO2, index);
						LOAD_CC_STEPX(TOPOO2, index);
						LOAD_CC_SETTING(TOPOO2, index, 1);
						LOAD_CC_SETTING(TOPOO2, index, 2);
						LOAD_CC_SETTING(TOPOO2, index, 3);
						LOAD_CC_SETTING(TOPOO2, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOO2, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOO2);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOO2_StartX= CC_TOPO_XMIN;
						var_CC_TOPOO2_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOO2_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOO2, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOO2, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOO2, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOO2_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOO2_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOO2(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD read_CC_TOPOO3  -  read and parse CC file
// 
METHOD read_CC_TOPOO3
{
    LABEL    "Read file CCTOPOO3.BIN|de|Lesen Datei CCTOPOO3.BIN"; // [TXT_read_CC_TOPOO3];
	ACCESS    OFFLINE;
	TYPE    unsigned int;
    DEFINITION
    {
		long    lval;
	    unsigned int  datlen;
	    unsigned int  index;
	    unsigned int  n;
		unsigned int  rc;
		
		// init read mode 
		var_CC_mode= CC_HEADER;
		var_CC_ActiveTask= "Load data file CCTOPOO3";
	    var_CC_Progress= 0;
		var_CC_DataIndex= 0;
		////// FT_GET: send first Get request
		// PUT_MESSAGE ([TXT_FT_START_GET]);
		_TRACE("### --> read_CC_TOPOO3(): FT_GET= send first Get request");
		SC_prepare_FT_getfile(CC_TOPOO3_FILE);
		if (SC_OKAY != do_SC_REQ())
		{
			return (SC_ERROR);
		}
		// if FT_GET not successful --> EXIT
		switch (SC_get_FT_responsecode())
		{
		    default:
			{
				// PUT_MESSAGE ([TXT_FT_ABORT_1STGET]);
				_TRACE("### --> read_CC_TOPOO3(): 1st FT_GET FAILED");
				return (SC_ERROR);
				break;
			}
			case FT_RSP_NOT_FOUND:
			{
				_TRACE("### --> read_CC_TOPOO3(): file not found");
			    return (SC_NOT_FOUND);
			    break;
			}
			case FT_RSP_OK:
			case FT_RSP_CONTINUE:
			{
			    break;
			}
		}
		
		do
		{
			/* checking response code (rc is loop condition) */
			rc= SC_get_FT_responsecode();
			// break in case of error and
		    // check valid OBEX header for file data
			if ((rc == SC_ERROR) || ((var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_BODY) && (var_SC_RSP_Buffer[SC_START_FTHEAD2] != FT_OBEX_ENDOFBODY)))
			{
			    // FT_GETF not successful --> EXIT
				_TRACE("### --> read_CC_TOPOO3(): FT_GET FAILED");
				return (SC_ERROR);
			}
			else
			{
			    // length element in header includes 3 header bytes (big endian)
				// assumed structure of telegram data: <RSP_CODE><Len1hi><Len1lo><HID><Len2hi><Len2lo><Data0>...<Datan>, n = Len2 - 3
			    datlen= NTOH(var_SC_RSP_Buffer[SC_START_FTHEAD2+1], var_SC_RSP_Buffer[SC_START_FTHEAD2+2]);
				// set byte index to first body data byte
				index= SC_START_FTBODY;
				// evaluate and store received data
				switch (var_CC_mode)
				{
					default:
					case CC_HEADER:
					{
					    // assign header information (using function macros with concatenation)
						LOAD_CC_VERSION(TOPOO3, index);
					    LOAD_CC_TYPE(TOPOO3, index);
						LOAD_CC_NUMBER(TOPOO3, index);
						LOAD_CC_KNR(TOPOO3, index);
						LOAD_CC_WKN(TOPOO3, index);
						LOAD_CC_NAME(TOPOO3, index);
						LOAD_CC_TIME_STORED(TOPOO3, index);
						LOAD_CC_TIME_CREATED(TOPOO3, index);
						LOAD_CC_NUMVALUES(TOPOO3, index);
						LOAD_CC_BPV(TOPOO3, index);
						LOAD_CC_STARTX(TOPOO3, index);
						LOAD_CC_STEPX(TOPOO3, index);
						LOAD_CC_SETTING(TOPOO3, index, 1);
						LOAD_CC_SETTING(TOPOO3, index, 2);
						LOAD_CC_SETTING(TOPOO3, index, 3);
						LOAD_CC_SETTING(TOPOO3, index, 4);
						// load FW-Version (only with CC_VERSION 0x03xx)
						LOAD_CC_FW_VERSION(TOPOO3, index);
						// check correct version (--> exit at failure!)
						CHECK_CC_VERSION(TOPOO3);
						// adjust x-axis properties
						// x-axis from 0% .. 100%, step 0.1%
						var_CC_TOPOO3_StartX= CC_TOPO_XMIN;
						var_CC_TOPOO3_StepX= CC_TOPO_XSTEP;

						var_CC_mode= CC_DATA;
						
						/* @@@fallthrough@@@ break; */
					}
					case CC_DATA:
					{
					    // assign data information
						switch (var_CC_TOPOO3_BytesPerValue)
						{
						    default:
						    case 1:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA1(TOPOO3, index, n); 
						        }
								break;
							}
						    case 2:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA2(TOPOO3, index, n); 
						        }
								break;
							}
						    case 4:
							{
					            for (n= var_CC_DataIndex; index < (datlen+SC_START_FTHEAD2); n++)
						        {
							        LOAD_CC_DATA4(TOPOO3, index, n); 
						        }
								break;
							}
						}
						var_CC_DataIndex= n;
		                var_CC_Progress= 100*n/var_CC_TOPOO3_NumValues;
						PUT_MESSAGE (itoa(n) + "/" + itoa(var_CC_TOPOO3_NumValues) + " (" + itoa(var_CC_Progress) + "%) read");
						break;
					}
				}
				
				// prepare FT_GETF
				_TRACE("### --> read_CC_TOPOO3(): FT_GETF= send Get request (after " + itoa(var_CC_DataIndex) + " data items)");
				SC_prepare_FT_getf();
				if (SC_OKAY != do_SC_REQ())
				{
					return (SC_ERROR);
				}
			}
		} while (rc == FT_RSP_CONTINUE);

		// success save and exit
        save_values();
		return (SC_OKAY);

	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
