//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2012). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Imported data:
//  Name....................: Doors Export P56
//  DeviceId................: unknown
//  DeviceName..............: AC2
//  Description.............: Baseline: 1.39 (FW3.0.0-xx), BudingV, 18 February 2011, AC01.2 Firmware 03.00.00-xx
//  RequestedFirmwareVersion: 0.23
//  SchemaVersion...........: 8.00
//  Timestamp...............: 18.02.2011 10:57:17
//  Version.................: 01.39
//  Author..................: Thomas Gutmann
//---------------------------------------------------------------------------------------------------------------------
// Generated 12.07.2012 13:45:45 by EDD-Generator Version 01.00.09.00
//  Selected root menu: Hauptmenü
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

#define PATH_DEVICE_NOT_FOUND  ""
#define PATH_AUMA_BASE          "\\ACTUATOR\\ELECTRIC\\AUMA\\"

#define FW_VERSION_INVALID      "0000"
#define FW_VERSION_DEFAULT_AC2  "0300"
#define FW_VERSION_DEFAULT_SXX  "0105"

#ifndef _V510_
#define LOG_MESSAGE(priority, text) _TRACE(text) 
#endif

#ifndef ID_CHECK_ABORT
#define	ID_CHECK_ABORT				0
#define	ID_CHECK_SUCCESS			1
#define	ID_CHECK_WRITABLE			0
#define	ID_CHECK_WRITE_PROTECTED	1
#endif

#ifndef ID_FC_ERROR
#define	ID_FC_ERROR					3000
#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_profi_ident (internal use only)
// 
VARIABLE var_profi_ident	// PRM_3006 Profibus Ident Nr
{
    LABEL		[TXT_PRM_3006];
    CLASS		CONTAINED;
    TYPE		ENUMERATED (2)      // TYP_1093
    {
        DEFAULT_VALUE	3151;
        { 3151, [TXT_TYP_1094] }, // Ident Nr AC einkanalige Profibushardware
        { 3261, [TXT_TYP_1095] }, // Ident Nr AC zweikanalige Profibushardware
        { 3191, [TXT_TYP_1107] }  // Ident Nr P70 Profibushardware
    }
    HANDLING		READ;
}
COMMAND read_profi_ident
{
    SLOT		30;
    INDEX		6;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            var_profi_ident    // 2 Octets
        }
        REQUEST
        {
        }
    }
}
 
//------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_version_path
// 
VARIABLE var_version_path	// Pfadname von Geräteversion abgeleitet
{
    LABEL		"Device version";
    CLASS		LOCAL;
    TYPE		ASCII(20)
    {
        DEFAULT_VALUE	FW_VERSION_INVALID;
    }
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_fw_version
// 
VARIABLE var_fw_version	// PRM_1759 Firmware Version
{
    LABEL		[TXT_PRM_1759];
    CLASS		CONTAINED;
    TYPE		ASCII(20)
    {
        DEFAULT_VALUE	"Vxx.xx.xx";
    }
	POST_READ_ACTIONS
	{
		extract_fw_version(var_fw_version, var_version_path),
		DEFINITION{ var_version_path.VARIABLE_STATUS = var_fw_version.VARIABLE_STATUS; }
	}
    HANDLING		READ;
}
COMMAND read_fw_version
{
    SLOT		17;
    INDEX		59;
    OPERATION	READ;
    TRANSACTION
    {
        REPLY
        {
            var_fw_version    // 20 Octets
        }
        REQUEST
        {
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  ret_DetectDevice
// 
VARIABLE ret_DetectDevice
{
	LABEL		"ret_DetectDevice";
	CLASS		LOCAL;
	TYPE		ASCII(255)
	{
		DEFAULT_VALUE PATH_DEVICE_NOT_FOUND;
	}
	HANDLING	READ;
}


//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD extract_fw_version
// 
// INPAR in: version string from device variable (e.g. "03.01.02-1234")
// OUTPAR out: extracted version string (e.g. "0301")
//
// TODO: parse and check version string: "[.+](\d\d)\.(\d\d)\.(\d\d)-(\d+)"
//       \1= major, \2= minor, \3= release, \4= build
//       and setup extracted version: "(\d\d)(\d\d)"
//       \1= major, \2= minor
#ifndef MAX_VERSION_LENGTH
#define MAX_VERSION_LENGTH    4
#endif
METHOD extract_fw_version (DD_STRING input, DD_STRING & output)
{
	LABEL		"extract_fw_version";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int i;
		int j;
		char vers[8];
		char in[20];
		in= input;
		vers= "";
		j= 0;
		for (i= 0; i < strlen(in) && j < MAX_VERSION_LENGTH; i++)
		{
			if (in[i] >= '0' && in[i] <= '9')
			{
				vers[j++]= in[i];
			}
		}
		if (strlen(vers) == 4)
		{
			output= vers;
		}
		else
		{
			output= FW_VERSION_INVALID;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD std_DetectDevice
// 
METHOD std_DetectDevice
{
	LABEL		"Detect device";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int		iResult;
		char	loc_DetectDevice[];

		loc_DetectDevice = PATH_DEVICE_NOT_FOUND;

		// LOG_MESSAGE(0, "### std_DetectDevice for AC2 is called");

		// Algorithmus:
		// * Lesen PROFIBUS-Ident (PRM_3006)
		// * Lesen Firmware-Version (PRM_1759) + extrahieren Major-Minor-Version ("[<V>]<MM>.<mm>.<rr>-<bbbbb>" --> "<MM><mm>")
		// * Zuweisen Ladepfad der EDD in Abhängigkeit von Ident und Version
		
		iResult = ID_FC_ERROR;

		iResult = ReadCommand(read_profi_ident); // profibus ident
		if (iResult < ID_FC_ERROR)
		{
			iResult = ReadCommand(read_fw_version); // version
			if (iResult < ID_FC_ERROR)
			{

				// LOG_MESSAGE(0, "### std_DetectDevice (input):  ident= " + itoa(var_profi_ident));
				// LOG_MESSAGE(0, "### std_DetectDevice (input):  version= " + var_fw_version + "(version path= \"V" + var_version_path + "\")");

				switch (var_profi_ident)
				{
					default:
					// AUMATIC AC1 (0x05CD ==> PATH_AUMA_BASE + "MATIC_VARIOMATIC\\")
				    {
						loc_DetectDevice = PATH_DEVICE_NOT_FOUND;
						break;
					}
					// AUMATIC AC2 (DPV1)
					case 0x0C4F:
					{
						if (strcmp(var_version_path, FW_VERSION_DEFAULT_AC2) >= 0)
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0C4F\\V" + var_version_path + "\\";
						}
						else
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0C4F\\V" + FW_VERSION_DEFAULT_AC2 + "\\";
						}
						break;
					}
					// AUMATIC AC2 (DPV1/V2)
					case 0x0CBD:
					{
						if (strcmp(var_version_path, FW_VERSION_DEFAULT_AC2) >= 0)
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0CBD\\V" + var_version_path + "\\";
						}
						else
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0CBD\\V" + FW_VERSION_DEFAULT_AC2 + "\\";
						}
						break;
					}
					// SGC/SVC
					case 0x0C77:
					{
						if (strcmp(var_version_path, FW_VERSION_DEFAULT_SXX) >= 0)
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0C77\\V" + var_version_path + "\\";
						}
						else
						{
							loc_DetectDevice = PATH_AUMA_BASE + "0C77\\V" + FW_VERSION_DEFAULT_SXX + "\\";
						}
						break;
					}
				}
			}
			else
			{
				LOG_MESSAGE(0, "### std_DetectDevice (ERROR):  Comm error while reading device version");
				process_abort();
			}
		}
		else
		{
			LOG_MESSAGE(0, "### std_DetectDevice (ERROR):  Comm error while reading device ident");
			process_abort();
		}
		ret_DetectDevice = loc_DetectDevice;
		// LOG_MESSAGE(0, "### std_DetectDevice (output):  ret_DetectDevice= " + ret_DetectDevice);
	}
}
