//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2011). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
// Description:
// * Definitions for ServiceChannel tunneled over DPV1
//
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "crc16.inc"
#include "sccom.inc"

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///*** Header formats
#define SC_HDR_STD                  0
#define SC_HDR_EXT                  1

///*** Function classes
#define SC_FCL_CMD_PRM              0x01
#define SC_FCL_CMD_PD               0x02
#define SC_FCL_CMD_CHAN             0x03
#define SC_FCL_CMD_MOVE             0x04
#define SC_FCL_CMD_PW               0x05
#define SC_FCL_CMD_FILE             0x06
#define SC_FCL_CMD_SRV              0x07
#define SC_FCL_CMD_REFRESH          0x7F
#define SC_FCL_CMD_INVALID          0xFF

///*** Function code (SC_FCL_CMD_PRM)
#define SC_FCO_PRM_READ             0x01
#define SC_FCO_PRM_WRITE            0x02
#define SC_FCO_PRM_READ_HEADER      0x03
#define SC_FCO_PRM_READ_ELDEF       0x04
#define SC_FCO_PRM_CONSIST_CHECK    0x05
#define SC_FCO_PRM_CONSIST_OFF      0x06
#define SC_FCO_PRM_CONSIST_ON       0x07
#define SC_FCO_PRM_RESTORE_SETTINGS 0x08
#define SC_FCO_PRM_SAVE_SETTINGS    0x09
#define SC_FCO_PRM_READ_ELDEFINX    0x0A
#define SC_FCO_PRM_SET_ACT2NOM      0x0B

///*** Function code (SC_FCL_CMD_PD + SC_FCL_CMD_CHAN)
#define SC_FCO_PD_READ              0x01
#define SC_FCO_PD_READ_HEADER       0x02
#define SC_FCO_PD_READ_ELDEF        0x03
#define SC_FCO_PD_READ_ELDEFINX     0x04
#define SC_FCO_PD_GLOBAL_SIM_ON     0x05
#define SC_FCO_PD_GLOBAL_SIM_OFF    0x06
#define SC_FCO_PD_LOCAL_SIM_ON      0x07
#define SC_FCO_PD_LOCAL_SIM_OFF     0x08
#define SC_FCO_PD_ALL_LOCAL_SIM_ON  0x09
#define SC_FCO_PD_ALL_LOCAL_SIM_OFF 0x0A
#define SC_FCO_PD_SIM_UPDATE        0x0B
#define SC_FCO_PD_SIM_TRIGGER       0x0C
#define SC_FCO_PD_SIM_TRIGUPDATE    0x0D
#define SC_FCO_PD_SIM_SETDELAY_MS   0x0E
#define SC_FCO_PD_MON_ACTIVATE      0x0F
#define SC_FCO_PD_MON_DEACTIVATE    0x10
#define SC_FCO_PD_MON_UPDATE        0x11

///*** Function code (SC_FCL_CMD_SRV)
#define SC_FCO_SRV_FILE             0x01
#define SC_FCO_SRV_SET_RTC          0x01
#define SC_FCO_SRV_GET_RTC          0x02
#define SC_FCO_SRV_RESET            0x03
#define SC_FCO_SRV_TEST_CAN         0x04
#define SC_FCO_SRV_LCD_ON           0x05
#define SC_FCO_SRV_LCD_OFF          0x06
#define SC_FCO_SRV_TEST_BUS1        0x07
#define SC_FCO_SRV_SAVE_LOGFILE     0x08
#define SC_FCO_SRV_TEST_BUS2        0x09
#define SC_FCO_SRV_COPY_DOWNLOAD    0x0A
#define SC_FCO_SRV_GET_SLAVEID      0x0B

///*** Function code (SC_FCL_CMD_MOVE)
#define SC_FCO_MV_REQLP              0x01
#define SC_FCO_MV_GETMOTIONSTATUS   0x02
#define SC_FCO_MV_RELLP             0x03
#define SC_FCO_MV_OPEN              0x04
#define SC_FCO_MV_CLOSE             0x05
#define SC_FCO_MV_STOP              0x06
#define SC_FCO_MV_RESET             0x07

///*** Function code (SC_FCL_CMD_PW)
#define SC_FCO_PW_GET               0x01
#define SC_FCO_PW_SET               0x02
#define SC_FCO_PW_RESET             0x03

///*** Function code (SC_FCL_CMD_FILE)
#define SC_FCO_FILE_OBEX_FTP        0x01

///*** Function code (FCL_CMD_INVALID)
#define SC_FCO_INVALID_FCL          0x01
#define SC_FCO_INVALID_FCO          0x02 

///*** Data type defines
#define SC_DTYPE_INT08              0
#define SC_DTYPE_UINT08             1
#define SC_DTYPE_INT16              2
#define SC_DTYPE_UINT16             3
#define SC_DTYPE_ENUM               SC_DTYPE_UINT16
#define SC_DTYPE_INT32              4
#define SC_DTYPE_UINT32             5
#define SC_DTYPE_STRING10           6
#define SC_DTYPE_STRING20           7
#define SC_DTYPE_STRING30           8
#define SC_DTYPE_STRING40           9
#define SC_DTYPE_BOOL               10
#define SC_DTYPE_OCTET4             13
#define SC_DTYPE_OCTET8             14
#define SC_DTYPE_OCTET16            15
#define SC_DTYPE_OCTET24            16
#define SC_DTYPE_OCTET32            17
#define SC_DTYPE_OCTET48            18
#define SC_DTYPE_OCTET64            19

///*** Return codes
#define SC_RC_ERR_WRONG_TELE_LEN    0xFF
#define SC_RC_ERR_FCO_NOT_SUPPORTED 0xFE
#define SC_RC_ERR_ACCESS_DENIED     0xFD
#define SC_RC_ERR_COMM_FAILED       0xFC
#define SC_RC_ACCEPTED              0xAA
#define SC_RC_DECLINED              0xDD

#define SC_RC_PARAM_OK              0
#define SC_RC_PARAM_ERR_SOURCE      1
#define SC_RC_PARAM_ERR_LEVEL       2
#define SC_RC_PARAM_ERR_NR          3
#define SC_RC_PARAM_ERR_INDEX       4
#define SC_RC_PARAM_ERR_PWD         5
#define SC_RC_PARAM_ERR_RANGE       6
#define SC_RC_PARAM_ERR_CONS        7
#define SC_RC_PARAM_ERR_BUSY        8
#define SC_RC_PARAM_ERR_UNDEF       9

///*** OBEX defines
#define FT_OBEX_VER                 0x10
#define FT_OBEX_FLG                 0x00
                                   
///*** OBEX header                 
#define FT_OBEX_BODY                0x48
#define FT_OBEX_ENDOFBODY           0x49

#define FT_FILE_SIZE_MAX            16960 // max. Größe LIST in PDM8 (?)


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VARIABLES
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining SC frame content variables (REQ)
var_SC_REQ_DST LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC REQ Header, DST"; }
var_SC_REQ_SRC LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC REQ Header, SRC"; }
var_SC_REQ_LEN LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC REQ Header, LEN"; }
var_SC_REQ_HDL LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC REQ Header, HDL"; }
var_SC_REQ_SRV LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC REQ Header, SRV"; }

VARIABLE var_SC_REQ_CRC
{
    LABEL		"SC REQ CRC";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(2)
	{
		EDIT_FORMAT		"04X";
		DISPLAY_FORMAT		"04X";
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  FT_REQ_SC_FCL
// 
VARIABLE var_SC_REQ_FCL
{
    LABEL		"SC REQ FCL";
    HELP		"Recent function class of SC telegram";
    CLASS		LOCAL;
    TYPE		ENUMERATED (1)
    {
		DEFAULT_VALUE	6;
		{ 1, "FCL_CMD_PRM" },
		{ 2, "FCL_CMD_PD" },
		{ 3, "FCL_CMD_CHAN" },
		{ 4, "FCL_CMD_MOVE" },
		{ 5, "FCL_CMD_PW" },
		{ 6, "FCL_CMD_FILE" },
		{ 7, "FCL_CMD_SRV" },
		{ 8, "FCL_CMD_INVALID" }
    }
    CONSTANT_UNIT		"";
    HANDLING		READ; // in Debug-Menü READ_ONLY, sonst HIDDEN
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_REQ_FCO
// 
VARIABLE var_SC_REQ_FCO
{
    LABEL		"SC REQ FCO";
    HELP		"Recent function code of SC telegram";
    CLASS		LOCAL;
    TYPE		ENUMERATED (1)
    {
		DEFAULT_VALUE	1;
		SELECT (var_SC_REQ_FCL ) 
		{
			CASE SC_FCL_CMD_PRM:
				{ 1, "FCO_PRM_READ" },
				{ 2, "FCO_PRM_WRITE" },
				{ 3, "FCO_PRM_READ_HEADER" },
				{ 4, "FCO_PRM_READ_ELDEF" },
				{ 5, "FCO_PRM_CONSIST_CHECK" },
				{ 6, "FCO_PRM_CONSIST_OFF" },
				{ 7, "FCO_PRM_CONSIST_ON" },
				{ 8, "FCO_PRM_RESTORE_SETTINGS" },
				{ 9, "FCO_PRM_SAVE_SETTINGS" },
				{ 10, "FCO_PRM_READ_ELDEFINX" },
				{ 11, "FCO_PRM_SET_ACT2NOM" }
			CASE SC_FCL_CMD_PD:
				{ 1, "FCO_PD_READ" },
				{ 2, "FCO_PD_READ_HEADER" },
				{ 3, "FCO_PD_READ_ELDEF" },
				{ 4, "FCO_PD_READ_ELDEFINX" },
				{ 5, "FCO_PD_GLOBAL_SIM_ON" },
				{ 6, "FCO_PD_GLOBAL_SIM_OFF" },
				{ 7, "FCO_PD_LOCAL_SIM_ON" },
				{ 8, "FCO_PD_LOCAL_SIM_OFF" },
				{ 9, "FCO_PD_ALL_LOCAL_SIM_ON" },
				{ 10, "FCO_PD_ALL_LOCAL_SIM_OFF" },
				{ 11, "FCO_PD_SIM_UPDATE" },
				{ 12, "FCO_PD_SIM_TRIGGER" },
				{ 13, "FCO_PD_SIM_TRIGUPDATE" },
				{ 14, "FCO_PD_SIM_SETDELAY_MS" },
				{ 15, "FCO_PD_MON_ACTIVATE" },
				{ 16, "FCO_PD_MON_DEACTIVATE" },
				{ 17, "FCO_PD_MON_UPDATE" }
			CASE SC_FCL_CMD_CHAN:
				{ 1, "FCO_PD_READ" },
				{ 2, "FCO_PD_READ_HEADER" },
				{ 3, "FCO_PD_READ_ELDEF" },
				{ 4, "FCO_PD_READ_ELDEFINX" },
				{ 5, "FCO_PD_GLOBAL_SIM_ON" },
				{ 6, "FCO_PD_GLOBAL_SIM_OFF" },
				{ 7, "FCO_PD_LOCAL_SIM_ON" },
				{ 8, "FCO_PD_LOCAL_SIM_OFF" },
				{ 9, "FCO_PD_ALL_LOCAL_SIM_ON" },
				{ 10, "FCO_PD_ALL_LOCAL_SIM_OFF" },
				{ 11, "FCO_PD_SIM_UPDATE" },
				{ 12, "FCO_PD_SIM_TRIGGER" },
				{ 13, "FCO_PD_SIM_TRIGUPDATE" },
				{ 14, "FCO_PD_SIM_SETDELAY_MS" },
				{ 15, "FCO_PD_MON_ACTIVATE" },
				{ 16, "FCO_PD_MON_DEACTIVATE" },
				{ 17, "FCO_PD_MON_UPDATE" }
			CASE SC_FCL_CMD_MOVE:
				{ 1, "FCO_MV_REQLP" },
				{ 2, "FCO_MV_GETMOTIONSTATUS" },
				{ 3, "FCO_MV_RELLP" },
				{ 4, "FCO_MV_OPEN" },
				{ 5, "FCO_MV_CLOSE" },
				{ 6, "FCO_MV_STOP" },
				{ 7, "FCO_MV_RESET" }
			CASE SC_FCL_CMD_PW:
				{ 1, "FCO_PW_GET" },
				{ 2, "FCO_PW_SET" },
				{ 3, "FCO_PW_RESET" }
			CASE SC_FCL_CMD_FILE:
				{ 1, "FCO_FILE_OBEX_FTP" }
			CASE SC_FCL_CMD_SRV:
				{ 1, "FCO_SRV_SET_RTC " },
				{ 2, "FCO_SRV_GET_RTC" },
				{ 3, "FCO_SRV_RESET" },
				{ 4, "FCO_SRV_TEST_CAN" },
				{ 5, "FCO_SRV_LCD_ON" },
				{ 6, "FCO_SRV_LCD_OFF" },
				{ 7, "FCO_SRV_TEST_BUS1" },
				{ 8, "FCO_SRV_SAVE_LOGFILE" },
				{ 9, "FCO_SRV_TEST_BUS2" },
				{ 10, "FCO_SRV_COPY_DOWNLOAD" },
				{ 11, "FCO_SRV_GET_SLAVEID" }
			CASE SC_FCL_CMD_INVALID:
				{ 1, "FCO_INVALID_FCL" },
				{ 2, "FCO_INVALID_FCO" }
		}
    }
    CONSTANT_UNIT	"";
    HANDLING		READ; // in Debug-Menü READ_ONLY, sonst HIDDEN
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining SC frame content variables (RSP)
var_SC_RSP_DST LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC RSP Header, DST"; }
var_SC_RSP_SRC LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC RSP Header, SRC"; }
var_SC_RSP_LEN LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC RSP Header, LEN"; }
var_SC_RSP_HDL LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC RSP Header, HDL"; }
var_SC_RSP_SRV LIKE VARIABLE var_SC_BYTE { REDEFINE LABEL   "SC RSP Header, SRV"; }

VARIABLE var_SC_RSP_CRC
{
    LABEL		"SC RSP CRC";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(2)
	{
		EDIT_FORMAT		"04X";
		DISPLAY_FORMAT		"04X";
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  FT_RSP_SC_FCL
// 
var_SC_RSP_FCL LIKE VARIABLE var_SC_REQ_FCL
{
	REDEFINE LABEL "SC RSP FCL";
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_RSP_FCO
// 
var_SC_RSP_FCO LIKE VARIABLE var_SC_REQ_FCO
{
	REDEFINE LABEL "SC RSP FCO";
    REDEFINE TYPE		ENUMERATED (1)
    {
		DEFAULT_VALUE	1;
		SELECT (var_SC_RSP_FCL) 
		{
			CASE SC_FCL_CMD_PRM:
				{ 1, "FCO_PRM_READ" },
				{ 2, "FCO_PRM_WRITE" },
				{ 3, "FCO_PRM_READ_HEADER" },
				{ 4, "FCO_PRM_READ_ELDEF" },
				{ 5, "FCO_PRM_CONSIST_CHECK" },
				{ 6, "FCO_PRM_CONSIST_OFF" },
				{ 7, "FCO_PRM_CONSIST_ON" },
				{ 8, "FCO_PRM_RESTORE_SETTINGS" },
				{ 9, "FCO_PRM_SAVE_SETTINGS" },
				{ 10, "FCO_PRM_READ_ELDEFINX" },
				{ 11, "FCO_PRM_SET_ACT2NOM" }
			CASE SC_FCL_CMD_PD:
				{ 1, "FCO_PD_READ" },
				{ 2, "FCO_PD_READ_HEADER" },
				{ 3, "FCO_PD_READ_ELDEF" },
				{ 4, "FCO_PD_READ_ELDEFINX" },
				{ 5, "FCO_PD_GLOBAL_SIM_ON" },
				{ 6, "FCO_PD_GLOBAL_SIM_OFF" },
				{ 7, "FCO_PD_LOCAL_SIM_ON" },
				{ 8, "FCO_PD_LOCAL_SIM_OFF" },
				{ 9, "FCO_PD_ALL_LOCAL_SIM_ON" },
				{ 10, "FCO_PD_ALL_LOCAL_SIM_OFF" },
				{ 11, "FCO_PD_SIM_UPDATE" },
				{ 12, "FCO_PD_SIM_TRIGGER" },
				{ 13, "FCO_PD_SIM_TRIGUPDATE" },
				{ 14, "FCO_PD_SIM_SETDELAY_MS" },
				{ 15, "FCO_PD_MON_ACTIVATE" },
				{ 16, "FCO_PD_MON_DEACTIVATE" },
				{ 17, "FCO_PD_MON_UPDATE" }
			CASE SC_FCL_CMD_CHAN:
				{ 1, "FCO_PD_READ" },
				{ 2, "FCO_PD_READ_HEADER" },
				{ 3, "FCO_PD_READ_ELDEF" },
				{ 4, "FCO_PD_READ_ELDEFINX" },
				{ 5, "FCO_PD_GLOBAL_SIM_ON" },
				{ 6, "FCO_PD_GLOBAL_SIM_OFF" },
				{ 7, "FCO_PD_LOCAL_SIM_ON" },
				{ 8, "FCO_PD_LOCAL_SIM_OFF" },
				{ 9, "FCO_PD_ALL_LOCAL_SIM_ON" },
				{ 10, "FCO_PD_ALL_LOCAL_SIM_OFF" },
				{ 11, "FCO_PD_SIM_UPDATE" },
				{ 12, "FCO_PD_SIM_TRIGGER" },
				{ 13, "FCO_PD_SIM_TRIGUPDATE" },
				{ 14, "FCO_PD_SIM_SETDELAY_MS" },
				{ 15, "FCO_PD_MON_ACTIVATE" },
				{ 16, "FCO_PD_MON_DEACTIVATE" },
				{ 17, "FCO_PD_MON_UPDATE" }
			CASE SC_FCL_CMD_MOVE:
				{ 1, "FCO_MV_REQLP" },
				{ 2, "FCO_MV_GETMOTIONSTATUS" },
				{ 3, "FCO_MV_RELLP" },
				{ 4, "FCO_MV_OPEN" },
				{ 5, "FCO_MV_CLOSE" },
				{ 6, "FCO_MV_STOP" },
				{ 7, "FCO_MV_RESET" }
			CASE SC_FCL_CMD_PW:
				{ 1, "FCO_PW_GET" },
				{ 2, "FCO_PW_SET" },
				{ 3, "FCO_PW_RESET" }
			CASE SC_FCL_CMD_FILE:
				{ 1, "FCO_FILE_OBEX_FTP" }
			CASE SC_FCL_CMD_SRV:
				{ 1, "FCO_SRV_SET_RTC " },
				{ 2, "FCO_SRV_GET_RTC" },
				{ 3, "FCO_SRV_RESET" },
				{ 4, "FCO_SRV_TEST_CAN" },
				{ 5, "FCO_SRV_LCD_ON" },
				{ 6, "FCO_SRV_LCD_OFF" },
				{ 7, "FCO_SRV_TEST_BUS1" },
				{ 8, "FCO_SRV_SAVE_LOGFILE" },
				{ 9, "FCO_SRV_TEST_BUS2" },
				{ 10, "FCO_SRV_COPY_DOWNLOAD" },
				{ 11, "FCO_SRV_GET_SLAVEID" }
			CASE SC_FCL_CMD_INVALID:
				{ 1, "FCO_INVALID_FCL" },
				{ 2, "FCO_INVALID_FCO" }
		}
    }
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLES for access control
// 
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_REQ_PW
// 
VARIABLE var_SC_REQ_PW
{
    LABEL		"SC REQ AccLevel";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(1);
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_RSP_PW
// 
var_SC_RSP_PW LIKE VARIABLE var_SC_REQ_PW
{
    REDEFINE LABEL		"SC RSP AccLevel";
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLES for PRM/PD communication
// 
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_REQ_ObjID
// 
VARIABLE var_SC_REQ_ObjID
{
    LABEL		"SC REQ ObjectID";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(4);
	WRITE_TIMEOUT	SC_TIMEOUT;
	READ_TIMEOUT	SC_TIMEOUT;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_RSP_ObjID
// 
var_SC_RSP_ObjID LIKE VARIABLE var_SC_REQ_ObjID
{
    REDEFINE LABEL		"SC RSP ObjectID";
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_REQ_ValU32
// 
VARIABLE var_SC_REQ_ValU32
{
    LABEL		"SC REQ Value (U32)";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER(4);
	WRITE_TIMEOUT	SC_TIMEOUT;
	READ_TIMEOUT	SC_TIMEOUT;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_SC_RSP_ObjID
// 
var_SC_RSP_ValU32 LIKE VARIABLE var_SC_REQ_ValU32
{
    REDEFINE LABEL		"SC RSP Value (U32)";
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLES for FT operation
// 
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_FT_REQ_OP_CODE
// 
VARIABLE var_FT_REQ_OP_CODE
{
    LABEL		"FT Operation";
    HELP		"Recent operation for file transfer|de|Aktuelle Operation der Datenübertragung";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (1)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT	"";
    HANDLING		READ; // in Debug-Menü READ_ONLY, sonst HIDDEN
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_FT_RSP_OP_CODE
// 
var_FT_RSP_OP_CODE LIKE VARIABLE var_FT_REQ_OP_CODE
{
    REDEFINE LABEL		"FT Response code";
    REDEFINE HELP		"Response code of recent operation for file transfer|de|Bewertung der letzten Operation der Datenübertragung";
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_FT_REQ_CONNID
// 
VARIABLE var_FT_REQ_CONNID
{
    LABEL		"FT REQ Conn-ID";
    HELP		"Connection id for file transfer|de|Verbindungs-Kennung der Datenübertragung";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (4)
    {
        DEFAULT_VALUE	0;
    }
    CONSTANT_UNIT	"";
    HANDLING		READ; // in Debug-Menü READ_ONLY, sonst HIDDEN
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_FT_RSP_CONNID
// 
var_FT_RSP_CONNID LIKE VARIABLE var_FT_REQ_CONNID
{
	REDEFINE LABEL "FT RSP Conn-ID";
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MENUS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   MENU	menu_SC_Request
// 
MENU	menu_SC_Request
{
	LABEL	"SC Tele (request)";
	ITEMS
	{
		var_SC_REQ_DST,
		var_SC_REQ_SRC,
		var_SC_REQ_LEN,
		var_SC_REQ_HDL,
		var_SC_REQ_SRV,
		var_SC_REQ_FCL,
		var_SC_REQ_FCO,
		var_SC_REQ_Buffer_SIZE,
		var_SC_REQ_CRC,
		var_SC_REQ_PW
	} 
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   MENU	menu_SC_Response
// 
MENU	menu_SC_Response
{
	LABEL	"SC Tele (response)";
	ITEMS
	{
		var_SC_RSP_DST,
		var_SC_RSP_SRC,
		var_SC_RSP_LEN,
		var_SC_RSP_HDL,
		var_SC_RSP_SRV,
		var_SC_RSP_FCL,
		var_SC_RSP_FCO,
		var_SC_RSP_Buffer_SIZE,
		var_SC_RSP_CRC,
		var_SC_RSP_PW
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD clear_SC_REQ_Buffer
// 
METHOD clear_SC_REQ_Buffer
{
	LABEL		"clear_SC_REQ_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		while (var_SC_REQ_Buffer.COUNT > 0)
		{
			ListDeleteElementAt(var_SC_REQ_Buffer, var_SC_REQ_Buffer.COUNT-1);
		}
		lassign(var_SC_REQ_Buffer_SIZE, var_SC_REQ_Buffer.COUNT);
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD malloc_SC_REQ_Buffer
// 
METHOD malloc_SC_REQ_Buffer (unsigned int size)
{
	LABEL		"malloc_SC_REQ_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		int i;
		// append elements to list
		for (i= 0; i < size; i++)
		{
            ListInsert(var_SC_REQ_Buffer, var_SC_REQ_Buffer.COUNT, var_SC_BYTE);
		}
		lassign(var_SC_REQ_Buffer_SIZE, var_SC_REQ_Buffer.COUNT);
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD resize_SC_REQ_Buffer
// 
METHOD resize_SC_REQ_Buffer (unsigned int new_size)
{
	LABEL		"resize_SC_REQ_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		int i;
		unsigned int size;
		size= var_SC_REQ_Buffer.COUNT;
		if (new_size > size)
		{
			// append elemtents to list
			for (i= size; i < new_size; i++)
			{
				ListInsert(var_SC_REQ_Buffer, i, var_SC_BYTE);
			}
		}
		else if (new_size < size)
		{
			while (var_SC_REQ_Buffer.COUNT > new_size)
			{
				ListDeleteElementAt(var_SC_REQ_Buffer, var_SC_REQ_Buffer.COUNT-1);
			}
		}
		lassign(var_SC_REQ_Buffer_SIZE, var_SC_REQ_Buffer.COUNT);
		save_values();
		_TRACE("### --> resize_SC_REQ_Buffer(): old size= " + itoa(size) + ", new size= " + itoa(var_SC_REQ_Buffer.COUNT));
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD clear_SC_RSP_Buffer
// 
METHOD clear_SC_RSP_Buffer
{
	LABEL		"clear_SC_RSP_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		while (var_SC_RSP_Buffer.COUNT > 0)
		{
			ListDeleteElementAt(var_SC_RSP_Buffer, var_SC_RSP_Buffer.COUNT-1);
		}
		lassign(var_SC_RSP_Buffer_SIZE, var_SC_RSP_Buffer.COUNT);
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD malloc_SC_RSP_Buffer
// 
METHOD malloc_SC_RSP_Buffer (unsigned int size)
{
	LABEL		"malloc_SC_RSP_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		int i;
		// append elemtents to list
		for (i= 0; i < size; i++)
		{
			ListInsert(var_SC_RSP_Buffer, var_SC_RSP_Buffer.COUNT, var_SC_BYTE);
		}
		lassign(var_SC_RSP_Buffer_SIZE, var_SC_RSP_Buffer.COUNT);
		save_values();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD resize_SC_RSP_Buffer
// 
METHOD resize_SC_RSP_Buffer (unsigned int new_size)
{
	LABEL		"resize_SC_RSP_Buffer";
	HELP		"Auxiliary method for buffer management";
	DEFINITION
	{
		int i;
		unsigned int size;
		size= var_SC_RSP_Buffer.COUNT;
		if (new_size > size)
		{
			// append elemtents to list
			for (i= size; i < new_size; i++)
			{
				ListInsert(var_SC_RSP_Buffer, i, var_SC_BYTE);
			}
		}
		else if (new_size < size)
		{
			while (var_SC_RSP_Buffer.COUNT > new_size)
			{
				ListDeleteElementAt(var_SC_RSP_Buffer, var_SC_RSP_Buffer.COUNT-1);
			}
		}
		lassign(var_SC_RSP_Buffer_SIZE, var_SC_RSP_Buffer.COUNT);
		save_values();
		_TRACE("### --> resize_SC_RSP_Buffer(): old size= " + itoa(size) + ", new size= " + itoa(var_SC_RSP_Buffer.COUNT));
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD finalize_SC_REQ_Buffer
//   - scan buffer, calculate crc, append crc at end of buffer
// 
METHOD finalize_SC_REQ_Buffer
{
	LABEL		"finalize_SC_REQ_Buffer";
	DEFINITION
	{
		unsigned int count;
		unsigned int i;
		unsigned int ui16Crc;
		
		ui16Crc= 0;
		count= var_SC_REQ_Buffer.COUNT-2;
		for (i= 0; i < count; i++)
		{
			ui16Crc = crc16_update(ui16Crc, var_SC_REQ_Buffer[i]);
		}
		// append crc to list (byte swapped)
		var_SC_REQ_Buffer[count]= LoByte(ui16Crc);
		var_SC_REQ_Buffer[count+1]= HiByte(ui16Crc);
		// refresh variables
		lassign(var_SC_REQ_DST, var_SC_REQ_Buffer[0]);
		lassign(var_SC_REQ_SRC, var_SC_REQ_Buffer[1]);
		lassign(var_SC_REQ_LEN, var_SC_REQ_Buffer[2]);
		lassign(var_SC_REQ_HDL, var_SC_REQ_Buffer[3]);
		lassign(var_SC_REQ_SRV, var_SC_REQ_Buffer[4]);
		lassign(var_SC_REQ_FCL, var_SC_REQ_Buffer[5]);
		lassign(var_SC_REQ_FCO, var_SC_REQ_Buffer[6]);
		lassign(var_SC_REQ_CRC, ui16Crc);
		save_values();
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD check_SC_RSP_Buffer
// 
METHOD check_SC_RSP_Buffer
{
	LABEL		"check_SC_RSP_Buffer";
	HELP		"Auxiliary method for service channel";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int count;
		unsigned int i;
        unsigned short ui16Crc;
        unsigned short ui16CrcBuf;
		
		ui16Crc= 0;
		// check for proper SC telegram length (5 SC-Head + 2 CRC)
		if (var_SC_RSP_Buffer.COUNT >= 7)
		{
			// last 2 bytes are crc
			count= var_SC_RSP_Buffer.COUNT-2;
			ui16CrcBuf= var_SC_RSP_Buffer[count] | var_SC_RSP_Buffer[count+1] << 8;
#ifdef WITH_RSP_CRC_CHECK
			for (i= 0; i < count; i++)
			{
				ui16Crc = (unsigned short)crc16_update(ui16Crc, var_SC_RSP_Buffer[i]);
			}
			_TRACE("### --> check_SC_RSP_Buffer(): calc_cs= " + itoa(ui16Crc) + ", recv_cs= " + itoa(ui16CrcBuf));
#else
			_TRACE("### --> check_SC_RSP_Buffer(): NO CRC check for RSP buffer");
			ui16Crc= ui16CrcBuf;
#endif
			// crc's shall be equal
			if (ui16Crc == ui16CrcBuf)
			{
				_TRACE("### --> check_SC_RSP_Buffer(): checksum ok.");
				// read frame
				lassign(var_SC_RSP_DST, var_SC_RSP_Buffer[0]);
				lassign(var_SC_RSP_SRC, var_SC_RSP_Buffer[1]);
				lassign(var_SC_RSP_LEN, var_SC_RSP_Buffer[2]);
				lassign(var_SC_RSP_HDL, var_SC_RSP_Buffer[3]);
				lassign(var_SC_RSP_SRV, var_SC_RSP_Buffer[4]);
				if (count >= 7)
				{
					lassign(var_SC_RSP_FCL, var_SC_RSP_Buffer[5]);
					lassign(var_SC_RSP_FCO, var_SC_RSP_Buffer[6]);
				}
				else
				{
					lassign(var_SC_RSP_FCL, 0);
					lassign(var_SC_RSP_FCO, 0);
				}
				lassign(var_SC_RSP_Buffer_SIZE, var_SC_RSP_Buffer.COUNT);
				lassign(var_SC_RSP_CRC, ui16CrcBuf);
				save_values();
			}
			else
			{
				_TRACE("### --> check_SC_RSP_Buffer(): checksum failure");
				ui16Crc= 0;
			}
		}
		if (ui16Crc == 0)
		{
			PUT_MESSAGE ("Checksum failure in response packet");
			return (SC_ERROR);
		}
		else
		{
			return (SC_OKAY);
		}
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD do_SC_REQ
// 
METHOD do_SC_REQ
{
	LABEL		"do_SC_REQ";
	HELP		"Main method for handling service channel";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int count;
		count= 0;
		do 
		{
			// if SC_REFRESH_MAX exceeded... return
			if (count++ > SC_REFRESH_MAX)
			{
				PUT_MESSAGE("do_SC_REQ: " + itoa(SC_REFRESH_MAX) + " REFRESH cycles exceeded --> exit!");
				return (SC_ERROR);
			}
			// METHOD requires prepared REQ buffer
			// 1st step: finalizing
			finalize_SC_REQ_Buffer();
			// send request
			if (SC_OKAY != send_SC_Request())
			{
				PUT_MESSAGE("do_SC_REQ: send request failed");
				return (SC_ERROR);
			}
			// read response
			if (SC_OKAY != receive_SC_Response())
			{
				PUT_MESSAGE("do_SC_REQ: receive response failed");
				return (SC_ERROR);
			}
			// check response
			if (SC_OKAY != check_SC_RSP_Buffer())
			{
				PUT_MESSAGE ("do_SC_REQ: RSP buffer check failed");
				return (SC_ERROR);
			}
			// if server responds with state PNDing
			if (var_SC_RSP_SRV == SC_SRV_PND)
			{
				// prepare REFresh request
				SC_prepare_REFRESH ();
				// Wait 1 s
				_TRACE ("### --> do_SC_REQ(): refresh..pending...");
				//DELAY(1,"do_SC_REQ: pending...");
			}
		} while (var_SC_RSP_SRV == SC_SRV_PND);
		// exit OK --> RSP buffer can get parsed
		return (SC_OKAY);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_REQ_HDR
// 
//   set header of service channel packet 
// 
METHOD SC_prepare_REQ_buffer (unsigned int _ext, unsigned int _datlen, unsigned int _hdl)
{
	LABEL		"SC_prepare_REQ_buffer";
	HELP		"Auxiliary method for service channel";
	DEFINITION
	{
		// resize REQ buffer (5 octets header + payload + 2 octets checksum)
		resize_SC_REQ_Buffer(_datlen + 7);

		if (_ext == SC_HDR_STD)
		{
			// standard header format
			// DST
			var_SC_REQ_Buffer[0]= 0x01;
			// SRC
			var_SC_REQ_Buffer[1]= 0x00;
			// LEN
			var_SC_REQ_Buffer[2]= LoByte(_datlen);
		}
		else
		{
			// extended header format
			// DST
			var_SC_REQ_Buffer[0]= 0xFE;
			// SRC
			var_SC_REQ_Buffer[1]= LoByte(_datlen);
			// LEN
			var_SC_REQ_Buffer[2]= HiByte(_datlen);
		}
		// HDL 
		var_SC_REQ_Buffer[3]= _hdl;
		// SRV
		var_SC_REQ_Buffer[4]= SC_SRV_REQ;
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_REQ_fcn_header
// 
//   set header of service channel packet for file transfer
// 
METHOD SC_prepare_REQ_fcn_header (unsigned int fcl, unsigned int fco)
{
	LABEL		"SC_prepare_REQ_fcn_header";
	HELP		"Auxiliary method for service channel";
	DEFINITION
	{
        // 2 Octets FCL+FCO
		var_SC_REQ_Buffer[5]= fcl;
		var_SC_REQ_Buffer[6]= fco;
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS for SC handling
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_REFRESH
// 
//   prepare buffer for command REFRESH
// 
METHOD SC_prepare_REFRESH
{
	LABEL		"SC_prepare_REFRESH";
	HELP		"Method for repeat recent request";
	DEFINITION
	{
		// resize REQ buffer (5 octets header + FCL+FCO + 2 octets checksum)
		resize_SC_REQ_Buffer(9);
		// set extended header, len= 2 octets,  repeat last HANDLE
		var_SC_REQ_Buffer[0]= 0xFE;
		// SRC= LoByte(LEN)
		var_SC_REQ_Buffer[1]= 0x02;
		// LEN= HiByte(LEN)
		var_SC_REQ_Buffer[2]= 0x00;
		// HDL 
		var_SC_REQ_Buffer[3]= var_SC_RSP_HDL;
		// SRV
		var_SC_REQ_Buffer[4]= SC_SRV_REF;
		// FCL + FCO
		var_SC_REQ_Buffer[5]= SC_FCL_CMD_REFRESH;
		var_SC_REQ_Buffer[6]= SC_FCL_CMD_REFRESH;
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS for pw handling
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_PW_GET
// 
//   prepare buffer for command PW_GET
// 
METHOD SC_prepare_PW_GET
{
	LABEL		"SC_prepare_PW_GET";
	HELP		"Method for getting access level";
	DEFINITION
	{
		// set standard header, len= 4 octets, increment handle
		SC_prepare_REQ_buffer (SC_HDR_STD, 4, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_PW, SC_FCO_PW_GET);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_check_PW_GET
// 
//   check response after command PW_GET
// 
METHOD SC_check_PW_GET
{
	LABEL		"SC_check_PW_GET";
	HELP		"Method for getting access level";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		// check valid length and status
		if (var_SC_RSP_Buffer.COUNT >= 11) 
		{
			if (var_SC_RSP_LEN == 4 && var_SC_RSP_Buffer[8]==SC_RC_ACCEPTED)
			{
				var_SC_RSP_PW= var_SC_RSP_Buffer[7];
				rc= SC_OKAY;
			}
			else
			{
				var_SC_RSP_PW= var_SC_RSP_Buffer[8];
				rc= SC_ERROR;
			}
		}
		else
		{
			var_SC_RSP_PW= 0xFF;
			rc= SC_ERROR;
		}
		save_values();
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_PW_SET4
// 
//   prepare buffer for command PW_SET
// 
METHOD SC_prepare_PW_SET4
{
	LABEL		"SC_prepare_PW_SET4";
	HELP		"Method for setting access level";
	DEFINITION
	{
		// set standard header, len= 7 octets, increment handle
		SC_prepare_REQ_buffer (SC_HDR_STD, 7, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_PW, SC_FCO_PW_SET);
		// 4 - Specialist (0000)
		// data byte 0: level 
		var_SC_REQ_Buffer[7]= 0x04;
		// data byte 1..4: PW
		var_SC_REQ_Buffer[8]= 0x30;
		var_SC_REQ_Buffer[9]= 0x30;
		var_SC_REQ_Buffer[10]= 0x30;
		var_SC_REQ_Buffer[11]= 0x30;
	}
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_PW_SET6
// 
//   prepare buffer for command PW_SET
// 
METHOD SC_prepare_PW_SET6
{
	LABEL		"SC_prepare_PW_SET6";
	HELP		"Method for setting access level";
	DEFINITION
	{
		// set standard header, len= 7 octets, increment handle
		SC_prepare_REQ_buffer (SC_HDR_STD, 7, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_PW, SC_FCO_PW_SET);
		// 6 - AUMA (1159)
		// data byte 0: level 
		var_SC_REQ_Buffer[7]= 0x06;
		// data byte 1..4: PW
		var_SC_REQ_Buffer[8]= 0x31;
		var_SC_REQ_Buffer[9]= 0x31;
		var_SC_REQ_Buffer[10]= 0x35;
		var_SC_REQ_Buffer[11]= 0x39;
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_check_PW_SET
// 
//   check response after command PW_SET
// 
METHOD SC_check_PW_SET
{
	LABEL		"SC_check_PW_SET";
	HELP		"Method for setting access level";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		// check valid length and status
		if (var_SC_RSP_Buffer.COUNT >= 11) 
		{
			if (var_SC_RSP_LEN == 4 && var_SC_RSP_Buffer[8]==SC_RC_ACCEPTED)
			{
				var_SC_RSP_PW= var_SC_RSP_Buffer[7];
				rc= SC_OKAY;
			}
			else
			{
				var_SC_RSP_PW= var_SC_RSP_Buffer[8];
				rc= SC_ERROR;
			}
		}
		else
		{
			var_SC_RSP_PW= 0xFF;
			rc= SC_ERROR;
		}
		save_values();
		return (rc);
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS for PRM commands
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_PRM_WRITE
// 
//   prepare buffer for command PRM_WRITE (U?Int(08|16|32) + BOOL)
// 
METHOD SC_prepare_PRM_WRITE (unsigned int dtyp)
{
	LABEL		"SC_prepare_PRM_WRITE";
	HELP		"Method for PRM write";
	DEFINITION
	{
		int idx;
		unsigned int payload;
		unsigned long objid;
		unsigned long value;

		// DataType U32
		objid= var_SC_REQ_ObjID;
		value= var_SC_REQ_ValU32;
		if ((dtyp == SC_DTYPE_INT08) || (dtyp == SC_DTYPE_UINT08)) {
			/* 2 byte FCL + 2 byte dtyp + 4 byte ObjID + 1 byte res. + n byte data*/
			payload= 9 + 1;
		}
		else {
		    if ((dtyp == SC_DTYPE_INT16) || (dtyp == SC_DTYPE_UINT16)) {
		        /* 2 byte FCL + 2 byte dtyp + 4 byte ObjID + 1 byte res. + n byte data*/
		        payload= 9 + 2;
		    }
		    else {
			    /* 2 byte FCL + 2 byte dtyp + 4 byte ObjID + 1 byte res. + n byte data*/
		        payload= 9 + 4;
			}
		}
		
		// set standard header, len= payload octets, increment handle
		SC_prepare_REQ_buffer (SC_HDR_STD, payload, var_SC_RSP_HDL+1);
		// data byte 0..1: FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_PRM, SC_FCO_PRM_WRITE);
		// set buffer index after header
		idx= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		// data byte 2..3: DataType (little-endian)
		var_SC_REQ_Buffer[idx++]= dtyp & 0xFF;
		var_SC_REQ_Buffer[idx++]= (dtyp >> 8) & 0xFF;
		// data byte 4..7: ObjectID (little-endian)
		var_SC_REQ_Buffer[idx++]= (objid >>  0) & 0xFF;
		var_SC_REQ_Buffer[idx++]= (objid >>  8) & 0xFF;
		var_SC_REQ_Buffer[idx++]= (objid >> 16) & 0xFF;
		var_SC_REQ_Buffer[idx++]= (objid >> 24) & 0xFF;
		// data byte 8: reserved
		var_SC_REQ_Buffer[idx++]= 0 & 0xFF;
		// data byte 9..12: value (little-endian)
		do {
		    var_SC_REQ_Buffer[idx++]= value & 0xFF;
			value= value >> 8;
		} while (idx <= (payload + SC_L4HEAD_LEN));
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_check_PRM_WRITE
// 
//   check response after command PRM_WRITE
// 
METHOD SC_check_PRM_WRITE
{
	LABEL		"SC_check_PRM_WRITE";
	HELP		"Method for PRM write";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		// check valid length and status
		if (var_SC_RSP_Buffer.COUNT >= 16) 
		{
			var_SC_RSP_ObjID= (var_SC_RSP_Buffer[12] << 24) | (var_SC_RSP_Buffer[11] << 16) | (var_SC_RSP_Buffer[10] << 8 ) | var_SC_RSP_Buffer[9];
			if (var_SC_RSP_ObjID == var_SC_REQ_ObjID && var_SC_RSP_Buffer[13] == SC_RC_PARAM_OK)
			{
				rc= SC_OKAY;
			}
			else
			{
				rc= SC_ERROR;
			}
		}
		else
		{
			var_SC_RSP_ObjID= 0xFFFFFFFF;
			rc= SC_ERROR;
		}
		save_values();
		return (rc);
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS for SRV commands
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_SRV_SAVE_LOGFILE
// 
METHOD SC_prepare_SRV_SAVE_LOGFILE
{
	LABEL		"SC_prepare_SRV_SAVE_LOGFILE";
	HELP		"Method for issuing saving command";
	DEFINITION
	{
		// set standard header, len= 2 octets, increment handle
		SC_prepare_REQ_buffer (SC_HDR_STD, 2, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_SRV, SC_FCO_SRV_SAVE_LOGFILE);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_check_CMD_SRV
// 
METHOD SC_check_CMD_SRV
{
	LABEL		"SC_check_CMD_SRV";
	HELP		"Method for checking response after command";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;
		// check valid length and status
		if (var_SC_RSP_Buffer.COUNT >= 11) 
		{
			if (var_SC_RSP_LEN == 4 && var_SC_RSP_Buffer[8]==SC_RC_ACCEPTED)
			{
				rc= SC_OKAY;
			}
			else
			{
				rc= SC_ERROR;
			}
		}
		else
		{
			rc= SC_ERROR;
		}
		return (rc);
	}
}

// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// METHODS for file transfer
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_connect_header
// 
//   set header CONNECT request
// 
METHOD SC_append_FT_connect_header (unsigned int offset, unsigned int _len)
{
	LABEL		"SC_append_FT_connect_header";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		// append header to REQ buffer 
        // 7 Octets OBEX header CONNECT
		var_SC_REQ_Buffer[offset++]= FT_REQ_CONNECT;
		var_SC_REQ_Buffer[offset++]= HiByte(_len);
		var_SC_REQ_Buffer[offset++]= LoByte(_len);
		// OP_OPTconnect = {VER= 10, FLG= 00, MLEN= 00DE}
		var_SC_REQ_Buffer[offset++]= 0x10;
		var_SC_REQ_Buffer[offset++]= 0x00;
		var_SC_REQ_Buffer[offset++]= 0x00;
		var_SC_REQ_Buffer[offset++]= 0xDE;
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_disconnect_header
// 
//   set header DISCONNECT request
// 
METHOD SC_append_FT_disconnect_header (unsigned int offset, unsigned int _len)
{
	LABEL		"SC_append_FT_disconnect_header";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		// append header to REQ buffer 
		// 3 Octets OBEX header DISCONNECT
		var_SC_REQ_Buffer[offset++]= FT_REQ_DISCONNECT;
		var_SC_REQ_Buffer[offset++]= HiByte(_len);
		var_SC_REQ_Buffer[offset++]= LoByte(_len);
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_setpath_header
// 
//   set header SETPATH request
// 
METHOD SC_append_FT_setpath_header (unsigned int offset, unsigned int _len)
{
	LABEL		"SC_append_FT_setpath_header";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		// append header to REQ buffer 
        // 5 Octets OBEX header SETPATH
		var_SC_REQ_Buffer[offset++]= FT_REQ_SETPATH;
		var_SC_REQ_Buffer[offset++]= HiByte(_len);
		var_SC_REQ_Buffer[offset++]= LoByte(_len);
		// OP_OPTsetpath = {FLG= 02, CON= 00}
		var_SC_REQ_Buffer[offset++]= 0x02;
		var_SC_REQ_Buffer[offset++]= 0x00;
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_getf_header
// 
//   set header GET (final) request
// 
METHOD SC_append_FT_getf_header (unsigned int offset, unsigned int _len)
{
	LABEL		"SC_append_FT_getf_header";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		// append header to REQ buffer 
		// 3 Octets OBEX header GET
		var_SC_REQ_Buffer[offset++]= FT_REQ_GET_F;
		var_SC_REQ_Buffer[offset++]= HiByte(_len);
		var_SC_REQ_Buffer[offset++]= LoByte(_len);
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_FBGUID
// 
//   set folder browsing GUID HID= 0x46 (byte sequence)
// 
METHOD SC_append_FT_FBGUID (unsigned int offset)
{
	LABEL		"SC_append_FT_FBGUID";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int   datlen;
		
		// LEN= 3 byte header + 16 byte FBGUID
		datlen=  SC_LEN_FT_FBGUID_HEADER;

		// append header to REQ buffer 
		// HID= "Target" (byte sequence, length prefixed with 2 byte unsigned integer)
		var_SC_REQ_Buffer[offset++]= 0x46;
		// LEN= 19
		var_SC_REQ_Buffer[offset++]= (datlen >> 8) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (datlen     ) & 0xFF;
		// FB-GUID "F9 EC 7B C4 95 3C 11 D2 98 4E 52 54 00 DC 9E 09" (ASCII-String)
		var_SC_REQ_Buffer[offset++]= 0xF9;
		var_SC_REQ_Buffer[offset++]= 0xEC; 
		var_SC_REQ_Buffer[offset++]= 0x7B; 
		var_SC_REQ_Buffer[offset++]= 0xC4; 
		var_SC_REQ_Buffer[offset++]= 0x95; 
		var_SC_REQ_Buffer[offset++]= 0x3C; 
		var_SC_REQ_Buffer[offset++]= 0x11; 
		var_SC_REQ_Buffer[offset++]= 0xD2; 
		var_SC_REQ_Buffer[offset++]= 0x98; 
		var_SC_REQ_Buffer[offset++]= 0x4E; 
		var_SC_REQ_Buffer[offset++]= 0x52; 
		var_SC_REQ_Buffer[offset++]= 0x54; 
		var_SC_REQ_Buffer[offset++]= 0x00; 
		var_SC_REQ_Buffer[offset++]= 0xDC; 
		var_SC_REQ_Buffer[offset++]= 0x9E; 
		var_SC_REQ_Buffer[offset++]= 0x09;
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_ConnID
// 
//   set connection ID
// 
METHOD SC_append_FT_ConnID (unsigned int offset, unsigned int connid)
{
	LABEL		"SC_append_FT_ConnID";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		// append header to REQ buffer 
		// HID= "Connection" (4 byte quantity -- transmitted in network byte order, high byte first)
		var_SC_REQ_Buffer[offset++]= 0xCB;
		var_SC_REQ_Buffer[offset++]= (connid >> 24) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (connid >> 16) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (connid >>  8) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (connid & 0xFF);
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_HID01
// 
//   set OBEX name header (UTF-16)
// 
METHOD SC_append_FT_HID01 (unsigned int offset, DD_STRING name)
{
	LABEL		"SC_append_FT_HID01";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		char  sz_name[];
		unsigned int   szlen;
		unsigned int   datlen;
		unsigned int   i;
		
		sz_name= name;
		szlen= strlen(sz_name);
		// LEN= 3 byte header + (szlen + 1) 2-byte-chars
		datlen=  3 + 2*(szlen + 1);

		// append header to REQ buffer 
		// HID= "Name" (null terminated Unicode text, length prefixed with 2 byte unsigned integer)
		var_SC_REQ_Buffer[offset++]= 0x01;
		var_SC_REQ_Buffer[offset++]= (datlen >> 8) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (datlen     ) & 0xFF;
		// name as UTF-16 (ASCII fills lowbyte)
		for (i= 0; i < szlen; i++)
		{
		    var_SC_REQ_Buffer[offset++]= 0x00;
		    var_SC_REQ_Buffer[offset++]= sz_name[i];
		}
		// append string termination (2 byte "\0")
		var_SC_REQ_Buffer[offset++]= 0x00;
		var_SC_REQ_Buffer[offset++]= 0x00;
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_append_FT_HID42
// 
//   set OBEX type file
// 
METHOD SC_append_FT_HID42 (unsigned int offset, DD_STRING type)
{
	LABEL		"SC_append_FT_HID42";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
	    char sz_type[];
		unsigned int  szlen;
		unsigned int  datlen;
		unsigned int  i;

		sz_type= type;
		szlen= strlen(sz_type);
		// LEN= 3 byte header + (szlen + 1) 1-byte-chars
		datlen=  3 + (szlen + 1);

		// append header to REQ buffer 
		// HID= "Type" (byte sequence, length prefixed with 2 byte unsigned integer)
		var_SC_REQ_Buffer[offset++]= 0x42;
		var_SC_REQ_Buffer[offset++]= (datlen >> 8) & 0xFF;
		var_SC_REQ_Buffer[offset++]= (datlen     ) & 0xFF;
		// name as ASCII 
		for (i= 0; i < szlen; i++)
		{
		    var_SC_REQ_Buffer[offset++]= sz_type[i];
		}
		// append string termination ("\0")
		var_SC_REQ_Buffer[offset++]= 0x00;
		return (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_FT_connect
// 
//   prepares connect for file transfer
// 
METHOD SC_prepare_FT_connect
{
	LABEL		"SC_prepare_FT_connect";
	HELP		"Auxiliary method for file transfer";
	DEFINITION
	{
		unsigned int offset;
		unsigned int datlen;

		datlen= SC_LEN_FT_FBGUID_HEADER + SC_LEN_FT_CONNECT_HEADER;
		// set SC FT packet header (datlen with additional FCL+FCO bytes)
		SC_prepare_REQ_buffer(SC_HDR_EXT, SC_L5HEAD_LEN + datlen, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_FILE, SC_FCO_FILE_OBEX_FTP);

		// append SC FT connect header
		offset= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		offset= SC_append_FT_connect_header(offset, datlen);
		// append FBGUID
		offset= SC_append_FT_FBGUID (offset);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_FT_disconnect
// 
//   prepares disconnect for file transfer
// 
METHOD SC_prepare_FT_disconnect
{
	LABEL		"SC_prepare_FT_disconnect";
	HELP		"Auxiliary method for file transfer";
	DEFINITION
	{
		unsigned int offset;
		unsigned int datlen;
		
		datlen= SC_LEN_FT_OBEX_HEADER;
		// set SC FT packet header (datlen with additional FCL+FCO bytes)
		SC_prepare_REQ_buffer(SC_HDR_EXT, SC_L5HEAD_LEN + datlen, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_FILE, SC_FCO_FILE_OBEX_FTP);

		// append SC FT disconnect header
		offset= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		offset= SC_append_FT_disconnect_header(offset, datlen);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_FT_setpath_LOGFILES
// 
//   prepares setpath for file transfer (LOGFILES)
// 
METHOD SC_prepare_FT_setpath_LOGFILES
{
	LABEL		"SC_prepare_FT_setpath_LOGFILES";
	HELP		"Auxiliary method for file transfer";
	DEFINITION
	{
		unsigned int offset;
		unsigned int datlen;
		char sz_name[];

		sz_name= "LOGFILES";
		// LEN(PATHNAME(UTF-16)) + LEN(OBEX_SETPATH)
		datlen = SC_LEN_FT_SETPATH_HEADER + SC_LEN_FT_OBEX_HEADER + 2*(strlen(sz_name)+1);
		// set SC FT packet header (datlen with additional FCL+FCO bytes)
		SC_prepare_REQ_buffer(SC_HDR_EXT, SC_L5HEAD_LEN + datlen, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_FILE, SC_FCO_FILE_OBEX_FTP);

		// append SC FT setpath header
		offset= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		offset= SC_append_FT_setpath_header(offset, datlen);
		// append path name for EL/CC
		offset= SC_append_FT_HID01(offset, sz_name);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_FT_getfile
// 
//   prepares get for file transfer (reading event log)
// 
METHOD SC_prepare_FT_getfile (DD_STRING fname)
{
	LABEL		"SC_prepare_FT_getfile";
	HELP		"Auxiliary method for file transfer";
	DEFINITION
	{
		unsigned int offset;
		unsigned int datlen;
		char sz_type[];

		sz_type= "x-obex/file";

		// LEN(FILENAME(UTF-16))
		datlen = SC_LEN_FT_OBEX_HEADER + 2*(strlen(fname)+1);
		// LEN(TYPENAME(ASCII))
		datlen += SC_LEN_FT_OBEX_HEADER + strlen(sz_type)+1;
		// LEN(CONNID)
		datlen += SC_LEN_FT_OBEX_HEADER + SC_LEN_FT_CONN_ID_HEADER;
		// set SC FT packet header (datlen with additional FCL+FCO bytes)
		SC_prepare_REQ_buffer(SC_HDR_EXT, SC_L5HEAD_LEN + datlen, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_FILE, SC_FCO_FILE_OBEX_FTP);

		// append SC FT getf header
		offset= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		offset= SC_append_FT_getf_header(offset, datlen);
		// append Connection ID to buffer variable
		lassign(var_FT_REQ_CONNID, 0);
		offset= SC_append_FT_ConnID(offset, var_FT_REQ_CONNID);

		// append OBEX header TYPE to buffer variable ()
		offset= SC_append_FT_HID42 (offset,  sz_type);

		// append OBEX header NAME to buffer variable
		offset= SC_append_FT_HID01 (offset, fname);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_prepare_FT_getf
// 
//   prepares simple getf cmd for file transfer
// 
METHOD SC_prepare_FT_getf
{
	LABEL		"SC_prepare_FT_getf";
	HELP		"Auxiliary method for file transfer";
	DEFINITION
	{
		unsigned int offset;
		// set SC FT packet header
		SC_prepare_REQ_buffer(SC_HDR_EXT, SC_LEN_FT_GET, var_SC_RSP_HDL+1);
		// FCL + FCO
		SC_prepare_REQ_fcn_header (SC_FCL_CMD_FILE, SC_FCO_FILE_OBEX_FTP);

		// append SC FT getf header
		offset= SC_L4HEAD_LEN + SC_L5HEAD_LEN;
		offset= SC_append_FT_getf_header(offset, SC_LEN_FT_GET_HEADER);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_check_FT_response
// 
//   check and analyze response packet of file transfer
// 
METHOD SC_check_FT_response
{
	LABEL		"SC_check_FT_response";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;

		// has RSP buffer sufficient length (5+2 byte SC header + 2 byte crc + 3 byte OBEX header = 12 byte)
		if (var_SC_RSP_Buffer.COUNT >= 12)
		{
			// calculate success of operation
			if ((var_SC_RSP_Buffer[7] == FT_RSP_CONTINUE) || (var_SC_RSP_Buffer[7] == FT_RSP_OK))
			{
			    rc= SC_OKAY;
			}
			else if (var_SC_RSP_Buffer[7] == FT_RSP_NOT_FOUND)
			{
			    rc= SC_NOT_FOUND;
			}
		}
		else
		{
			_TRACE ("### --> SC_check_FT_response(): to few bytes in response buffer!");
			rc= SC_ERROR;
		}
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD SC_get_FT_responsecode
// 
//   get response code of file transfer
// 
METHOD SC_get_FT_responsecode
{
	LABEL		"SC_get_FT_responsecode";
	HELP		"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		unsigned int rc;

		// has RSP buffer sufficient length (5+2 byte SC header + 2 byte crc + 3 byte OBEX header = 12 byte)
		if (var_SC_RSP_Buffer.COUNT >= 12)
		{
			rc= var_SC_RSP_Buffer[7];
		}
		else
		{
			_TRACE ("### --> SC_get_FT_responsecode(): to few bytes in response buffer!");
			rc= SC_ERROR;
		}
		return (rc);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD connect_FT_LOGFILES
// 
//   prepares file transfer (setting up folder LOGFILES)
// 
METHOD connect_FT_LOGFILES
{
	LABEL	"connect_FT_LOGFILES";
	HELP	"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		////// PW_SET: set appropriate access level
		_TRACE("### --> connect_FT_LOGFILES(): PW_SET= set appropriate access level");
		SC_prepare_PW_SET6();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if PW_SET not successful --> EXIT
		if (SC_OKAY != SC_check_PW_SET())
		{
			_TRACE("### --> connect_FT_LOGFILES(): PW_SET FAILED");
			return (SC_ERROR);
		}

		////// SRV_SAVE_LOGFILE: save logfile
		_TRACE("### --> connect_FT_LOGFILES(): SRV_SAVE_LOGFILE= save logfile");
		SC_prepare_SRV_SAVE_LOGFILE();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if SRV_SAVE_LOGFILE not successful --> EXIT
		if (SC_OKAY != SC_check_CMD_SRV())
		{
			_TRACE("### --> connect_FT_LOGFILES(): SRV_SAVE_LOGFILE not successful");
			// only warning: return (SC_ERROR);
		}

		////// FT_CONNECT: Initiate connection
		// prepare CONNECT for LOGFILES
		_TRACE("### --> connect_FT_LOGFILES(): FT_CONNECT= Initiate connection");
		SC_prepare_FT_connect();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if FT_CONNECT not successful --> EXIT
		if (SC_OKAY != SC_check_FT_response())
		{
			_TRACE("### --> connect_FT_LOGFILES(): FT_CONNECT FAILED");
			return (SC_ERROR);
		}

		////// FT_DISCONNECT: Cancel connection (FT server changes to root directory)
		// prepare DISCONNECT 
		_TRACE("### --> connect_FT_LOGFILES(): FT_DISCONNECT= Cancel connection (FT server changes to root directory)");
		SC_prepare_FT_disconnect();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if FT_DISCONNECT not successful --> EXIT
		if (SC_OKAY != SC_check_FT_response())
		{
			_TRACE("### --> connect_FT_LOGFILES(): FT_DISCONNECT FAILED");
			return (SC_ERROR);
		}

		////// FT_CONNECT: Re-establish connection
		// prepare CONNECT for EL
		_TRACE("### --> connect_FT_LOGFILES(): FT_CONNECT= Re-establish connection");
		SC_prepare_FT_connect();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if FT_CONNECT not successful --> EXIT
		if (SC_OKAY != SC_check_FT_response())
		{
			_TRACE("### --> connect_FT_LOGFILES(): FT_(RE)CONNECT FAILED");
			return (SC_ERROR);
		}

		////// FT_SETPATH: Set target directory
		// prepare SETPATH for EL
		_TRACE("### --> connect_FT_LOGFILES(): FT_SETPATH= Set target directory");
		SC_prepare_FT_setpath_LOGFILES();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): Send request failed");
			return (SC_ERROR);
		}
		// if FT_SETPATH not successful --> EXIT
		if (SC_OKAY != SC_check_FT_response())
		{
		    _TRACE("### --> connect_FT_LOGFILES(): FT_SETPATH FAILED");
			return (SC_ERROR);
		}
		return (SC_OKAY);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD disconnect_FT
// 
//   finishes file transfer 
// 
METHOD disconnect_FT
{
	LABEL	"disconnect_FT";
	HELP	"Auxiliary method for file transfer";
	TYPE    unsigned int;
	DEFINITION
	{
		////// FT_DISCONNECT: End connection
		// PUT_MESSAGE ([TXT_FT_DISCONNECT]);
		// prepare DISCONNECT 
		SC_prepare_FT_disconnect();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> disconnect_FT(): FT_DISCONNECT FAILED");
		    return (SC_ERROR);
		}
		// if FT_DISCONNECT not successful --> EXIT
		if (SC_OKAY != SC_check_FT_response())
		{
			_TRACE("### --> disconnect_FT(): FT_DISCONNECT FAILED");
			return (SC_ERROR);
		}
		_TRACE("### --> disconnect_FT(): FT_DISCONNECT succeeded");
		return (SC_OKAY);
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD do_SC_PRM_WRITE_PRIV_U32
// 
//   write a data object with SC (access level raised)
// 
METHOD do_SC_PRM_WRITE_PRIV (unsigned long objid, unsigned int dtyp, unsigned long value)
{
	LABEL	"do_SC_PRM_WRITE_PRIV";
	HELP	"Auxiliary method for parameter access";
	TYPE    unsigned int;
	DEFINITION
	{
	    if ((dtyp > SC_DTYPE_UINT32) && (dtyp != SC_DTYPE_BOOL)) {
			////// wrong argument
			_TRACE("### --> do_SC_PRM_WRITE_PRIV(): Failure wrong argument!");
			return (SC_ERROR);
		}
		////// PW_SET: set appropriate access level
		_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PW_SET= set appropriate access level");
		SC_prepare_PW_SET6();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> do_SC_PRM_WRITE_PRIV(): Send request failed");
			return (SC_ERROR);
		}
		// if PW_SET not successful --> EXIT
		if (SC_OKAY != SC_check_PW_SET())
		{
			_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PW_SET FAILED");
			return (SC_ERROR);
		}

		////// CMD_PRM PRM_WRITE
		_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PRM_PRM_WRITE");
		var_SC_REQ_ObjID= objid;
		var_SC_REQ_ValU32= value;
		SC_prepare_PRM_WRITE(dtyp);
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> do_SC_PRM_WRITE_PRIV(): Send request failed");
			return (SC_ERROR);
		}
		// if PRM_PRM_WRITE not successful --> EXIT
		if (SC_OKAY != SC_check_PRM_WRITE())
		{
			_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PRM_PRM_WRITE not successful");
			return (SC_ERROR);
		}
		////// PW_SET: re-set access level
		_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PW_SET= readjust access level");
		SC_prepare_PW_SET4();
		if (SC_OKAY != do_SC_REQ())
		{
		    _TRACE("### --> do_SC_PRM_WRITE_PRIV(): Send request failed");
			return (SC_ERROR);
		}
		// if PW_SET not successful --> EXIT
		if (SC_OKAY != SC_check_PW_SET())
		{
			_TRACE("### --> do_SC_PRM_WRITE_PRIV(): PW_SET FAILED");
			return (SC_ERROR);
		}
		return (SC_OKAY);
	}
}


// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// END OF FILE
// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
