/*---------------------------------------------------------------------------------------------------------------------
 * Copyright (c) AUMA Riester GmbH & Co. KG (2017). All Rights Reserved
 * Target System: SIMATIC PDM
 *---------------------------------------------------------------------------------------------------------------------
 * Description:
 * * Definitions for asset management functionality in edd 
 *
 *---------------------------------------------------------------------------------------------------------------------
 *  (c) Sipos Aktorik GmbH (Mathias Monse <mathias.monse@sipos.de>)
 *---------------------------------------------------------------------------------------------------------------------
 */

/*---------------------------------------------------------------------------------------------------------------------
 * DEFINE
 *---------------------------------------------------------------------------------------------------------------------
 */
#define AC2_TV

#define TV_MIN_POSITION   0.0
#define TV_MAX_POSITION   100.0
#define TV_POSITION_INCREMENT   1.0
/* Symbolisch: (1+(TV_MAX_POSITION - TV_MIN_POSITION)/TV_POSITION_INCREMENT) */
#define TV_POSITION_VECLEN  101
#define TV_POS_PZD_MAX     1000.0
#define TV_POS_PZD2SCOPE   ((TV_POSITION_VECLEN-1)/TV_POS_PZD_MAX)

#define K_Nm2FtLb     	(1/1.35581)
/*---------------------------------------------------------------------------------------------------------------------
 * VARIABLES
 *---------------------------------------------------------------------------------------------------------------------
 */
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   PZD_683_cmd
 */
VARIABLE PZD_683_cmd    /* Fahrbefehl-Feldbus (cmd) */
{
    LABEL       [TXT_TXT_149];
    HELP        [HLP_TXT_149];
    CLASS       DIAGNOSTIC;
    TYPE        ENUMERATED (2)      /* LTYP_10 */
    {
        DEFAULT_VALUE       0;
        { 0, [TXT_TXT_119], [HLP_TXT_119] } /* DrvCmd4_cmd_0 */
        ,{ 1, [TXT_TXT_120], [HLP_TXT_120] } /* DrvCmd4_cmd_1 */
        ,{ 2, [TXT_TXT_118], [HLP_TXT_118] } /* DrvCmd4_cmd_2 */
        ,{ 4, [TXT_TXT_117], [HLP_TXT_117] } /* DrvCmd4_cmd_4 */
    }
    HANDLING        READ;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   PZD_683_dir
 */
VARIABLE PZD_683_dir    /* Fahrbefehl-Feldbus (direction) */
{
    LABEL       [TXT_TXT_150];
    HELP        [HLP_TXT_150];
    CLASS       DIAGNOSTIC;
    TYPE        ENUMERATED (2)      /* LTYP_20 */
    {
        DEFAULT_VALUE       0;
        { 0, [TXT_TXT_151], [HLP_TXT_151] } /* DrvCmd4_dir_0 */
        ,{ 1, [TXT_TXT_152], [HLP_TXT_152] } /* DrvCmd4_dir_1 */
        ,{ 2, [TXT_TXT_153], [HLP_TXT_153] } /* DrvCmd4_dir_2 */
    }
    HANDLING        READ;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   PZD_683_speed
 */
VARIABLE PZD_683_speed  /* Fahrbefehl-Feldbus (speed) */
{
    LABEL       [TXT_TXT_157];
    HELP        [HLP_TXT_157];
    CLASS       DIAGNOSTIC;
    TYPE        UNSIGNED_INTEGER (2)
    {
        MAX_VALUE   1000;
        DEFAULT_VALUE   0;
        SCALING_FACTOR  0.1;
        EDIT_FORMAT ".1f";
        DISPLAY_FORMAT  ".1f";
    }
    CONSTANT_UNIT       "%";
    HANDLING        READ;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   PZD_683_position
 */
VARIABLE PZD_683_position   /* Fahrbefehl-Feldbus (position) */
{
    LABEL       [TXT_TXT_155];
    HELP        [HLP_TXT_155];
    CLASS       DIAGNOSTIC;
    TYPE        UNSIGNED_INTEGER (2)
    {
        MAX_VALUE   4095;
        DEFAULT_VALUE   0;
        SCALING_FACTOR  0.1;
        EDIT_FORMAT ".1f";
        DISPLAY_FORMAT  ".1f";
    }
    CONSTANT_UNIT       "%";
    HANDLING        READ;
}

COMMAND read_PZD_683
{
    SLOT        6;
    INDEX       211;
    OPERATION   READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_683_cmd  <0xE0000000>, PZD_683_dir  <0x18000000>, PZD_683_position  <0x00FFF000>, PZD_683_speed  <0x00000FFF>    /* 44-7: Logik: Fahrbefehl (4 Octets) */
        }
        REQUEST
        {
        }
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   PZD_1013
 */
VARIABLE PZD_1013   /* Beschleunigung gesamt (XYZ) */
{
    LABEL       "Acceleration total (XYZ)|de|Beschleunigung gesamt (XYZ)"; /* [TXT_PZD_1013];*/
    HELP        "";   /* [HLP_PZD_1013];*/
    CLASS       DIAGNOSTIC;
    TYPE        UNSIGNED_INTEGER (4)
    {
        MAX_VALUE   1000;
        DEFAULT_VALUE   0;
        SCALING_FACTOR  0.1;
        EDIT_FORMAT ".1f";
        DISPLAY_FORMAT  ".1f";
    }
    HANDLING        READ;
    VALIDITY    IF (((PRM_1752 != 0))) { TRUE; } ELSE { FALSE; }  /* (using RULE_62) */
}
COMMAND read_PZD_1013
{
    SLOT        10;
    INDEX       141;
    OPERATION   READ;
    TRANSACTION
    {
        REPLY
        {
            PZD_1013    /* 4 Octets */
        }
        REQUEST
        {
        }
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLEs var_TV_TorqueXXX
 *   var_TV_TorquePercent <=> (PZD_545-500)*100/127
 *   var_TV_TorqueNm   <=> (PZD_545-500)/127*TorqueNm_Ref
 *   var_TV_TorqueFtLb <=> TorqueNm * K_Nm2FtLb
 */
VARIABLE var_TV_TorquePercent 
{
    LABEL    "Torque [%]|de|Drehmoment [%]";
    CLASS   LOCAL;
    TYPE    FLOAT
	{
		DEFAULT_VALUE	0;
	}
    CONSTANT_UNIT   "%";
}
var_TV_TorqueNm LIKE VARIABLE var_TV_TorquePercent { REDEFINE LABEL "Torque [Nm]|de|Drehmoment [Nm]"; REDEFINE CONSTANT_UNIT "Nm"; }
var_TV_TorqueFtLb LIKE VARIABLE var_TV_TorquePercent { REDEFINE LABEL "Torque [ft-lb]|de|Drehmoment [ft-lb]"; REDEFINE CONSTANT_UNIT "ft-lb"; }

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLEs var_TV_TorqueXXX_Min
 */
VARIABLE var_TV_TorquePercent_Min
{
    LABEL    "Torque min|de|Drehmoment min";
    CLASS   LOCAL;
    TYPE    FLOAT
	{
		DEFAULT_VALUE	0;
	}
    CONSTANT_UNIT   "%";
}
var_TV_TorqueNm_Min LIKE VARIABLE var_TV_TorquePercent_Min { REDEFINE CONSTANT_UNIT "Nm";}
var_TV_TorqueFtLb_Min LIKE VARIABLE var_TV_TorquePercent_Min { REDEFINE CONSTANT_UNIT "ft-lb";}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   var_TV_TorqueXXX_Max
 */
var_TV_TorquePercent_Max LIKE VARIABLE var_TV_TorquePercent_Min { REDEFINE LABEL "Torque max|de|Drehmoment max";}
var_TV_TorqueNm_Max LIKE VARIABLE var_TV_TorquePercent_Max { REDEFINE CONSTANT_UNIT "Nm";}
var_TV_TorqueFtLb_Max LIKE VARIABLE var_TV_TorquePercent_Max { REDEFINE CONSTANT_UNIT "ft-lb";}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   var_TV_ActPos <=> INT32(PZD_2 * TV_POS_PZD2SCOPE)
 */
VARIABLE var_TV_ActPos 
{
    LABEL    "Actual Position|de|Istposition";
    CLASS   LOCAL;
    TYPE    FLOAT
	{
		DEFAULT_VALUE	0;
	}
    CONSTANT_UNIT   "0.1 %";
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   var_TV_SetPos <=> INT32(PZD_683.Pos * TV_POS_PZD2SCOPE)
 */
VARIABLE var_TV_SetPos 
{
    LABEL    "Set Position|de|Sollposition";
    CLASS   LOCAL;
    TYPE    FLOAT
	{
		DEFAULT_VALUE	0;
	}
    CONSTANT_UNIT   "0.1 %";
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   var_TV_PosIndex
 */
VARIABLE var_TV_PosIndex
{
    LABEL    "var_TV_PosIndex";
    CLASS   LOCAL;
    TYPE    INTEGER(2)
	{
		DEFAULT_VALUE	0;
	}
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE   var_TV_TrvlStatus <=> f(PZD_258, PZD_259, PZD_264, PZD_265, PZD_401)
 *      1: PZD_264==1 : Travel to OPEN
 *      2: PZD_265==1 : Travel to CLOSE
 *      3: PZD_258==1 : end position OPEN
 *      4: PZD_259==1 : end position CLOSE
 *      5: PZD_401==1 : set position E1 reached
 */
#define TV_TrvlStatus_MIN  0
#define TV_TrvlStatus_MAX  5
VARIABLE var_TV_TrvlStatus 
{
    LABEL    "Travel status|de|Fahrstatus";
    CLASS   LOCAL;
    TYPE    UNSIGNED_INTEGER(4)
	{
		DEFAULT_VALUE	0;
	}
}
 
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining ARRAY for torque values
 */ 
ARRAY arr_TV_Torque
{
    LABEL    "TV torque values";  /* [TXT_arr_TV_Torque]; */
    NUMBER_OF_ELEMENTS    TV_POSITION_VECLEN;
    TYPE    var_TV_TorqueNm;
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining ARRAY for vibration values
 */ 
ARRAY arr_TV_Vibration
{
    LABEL    "TV vibration values";  /* [TXT_arr_TV_Vibration]; */
    NUMBER_OF_ELEMENTS    TV_POSITION_VECLEN;
    TYPE    PZD_1013;
}
 
/* --------------------------------------------------------------------------------------------------------------------
 *  CHARTS
 * --------------------------------------------------------------------------------------------------------------------
 */
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_TorquePercent_yaxis
 */ 
AXIS    chart_TV_TorquePercent_yaxis
{
    LABEL   var_TV_TorquePercent.LABEL; 
    MIN_VALUE   var_TV_TorquePercent_Min;
    MAX_VALUE   var_TV_TorquePercent_Max;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_TorqueNm_yaxis
 */ 
AXIS    chart_TV_TorqueNm_yaxis
{
    LABEL   var_TV_TorqueNm.LABEL; 
    MIN_VALUE   var_TV_TorqueNm_Min;
    MAX_VALUE   var_TV_TorqueNm_Max;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_TorqueFtLb_yaxis
 */ 
AXIS    chart_TV_TorqueFtLb_yaxis
{
    LABEL   var_TV_TorqueFtLb.LABEL; 
    MIN_VALUE   var_TV_TorqueFtLb_Min;
    MAX_VALUE   var_TV_TorqueFtLb_Max;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_Pos_yaxis
 */ 
AXIS    chart_TV_Pos_yaxis
{
    LABEL   "Position|de|Stellung"; 
    MIN_VALUE   TV_MIN_POSITION;
    MAX_VALUE   TV_MAX_POSITION;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_TrvlStatus_yaxis
 */ 
AXIS    chart_TV_TrvlStatus_yaxis
{
    LABEL   var_TV_TrvlStatus.LABEL; 
    MIN_VALUE   TV_TrvlStatus_MIN;
    MAX_VALUE   TV_TrvlStatus_MAX;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_VibrationPerc_yaxis
 */ 
AXIS    chart_TV_VibrationPerc_yaxis
{
    LABEL   "Vibration [%]"; 
    MIN_VALUE   PZD_1013.MIN_VALUE;
    MAX_VALUE   PZD_1013.MAX_VALUE;
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining AXIS  chart_TV_VibrationG_yaxis
 */ 
AXIS    chart_TV_VibrationG_yaxis
{
    LABEL   "Vibration [g]"; 
    MIN_VALUE   PZD_174.MIN_VALUE;
    MAX_VALUE   PZD_174.MAX_VALUE;
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining CHART  chart_TV_Torque
 */ 
CHART chart_TV_Torque
{
    LABEL   "";
    TYPE    SCOPE;
    MEMBERS
    {
        SOURCE1, chart_source_TV_TorquePercent;
        SOURCE2, chart_source_TV_TorqueNm;
        SOURCE3, chart_source_TV_TorqueFtLb;
    }
}
SOURCE  chart_source_TV_TorquePercent
{
    LABEL   "";
    Y_AXIS  chart_TV_TorquePercent_yaxis;
    MEMBERS
    {
        VALUE1, var_TV_TorquePercent;
    }
    REFRESH_ACTIONS {
        method_TV_TorquePercent_refresh
    }
}
SOURCE  chart_source_TV_TorqueNm
{
    LABEL   "";
    Y_AXIS  chart_TV_TorqueNm_yaxis;
    MEMBERS
    {
        VALUE1, var_TV_TorqueNm;
    }
    INIT_ACTIONS    {
        method_TV_TorqueNm_init
    }
    REFRESH_ACTIONS {
        DEFINITION {
            method_TV_Pos_refresh();
            method_TV_TorqueNm_refresh();
        }
    }
}
SOURCE  chart_source_TV_TorqueFtLb
{
    LABEL   "";
    Y_AXIS  chart_TV_TorqueFtLb_yaxis;
    MEMBERS
    {
        VALUE1, var_TV_TorqueFtLb;
    }
    REFRESH_ACTIONS {
        method_TV_TorqueFtLb_refresh
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining CHART  chart_TV_Position
 */ 
CHART chart_TV_Position
{
    LABEL   "";
    TYPE    SCOPE;
    MEMBERS
    {
        SOURCE1, chart_source_TV_SetActPos;
    }
}
SOURCE  chart_source_TV_SetActPos
{
    LABEL   "";
    Y_AXIS  chart_TV_Pos_yaxis;
    MEMBERS
    {
        VALUE1, var_TV_SetPos;
        VALUE1, var_TV_ActPos;
    }
    REFRESH_ACTIONS {
        method_TV_Pos_refresh
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining CHART  chart_TV_Status
 */ 
CHART chart_TV_Status
{
    LABEL   "";
    TYPE    SCOPE;
    MEMBERS
    {
        SOURCE1, chart_source_TV_TrvlStatus;
    }
}
SOURCE  chart_source_TV_TrvlStatus
{
    LABEL   "";
    Y_AXIS  chart_TV_TrvlStatus_yaxis;
    MEMBERS
    {
        VALUE1, var_TV_TrvlStatus;
    }
    REFRESH_ACTIONS {
        method_TV_TrvlStatus_refresh
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining CHART  chart_TV_Vibration
 */ 
CHART chart_TV_Vibration
{
    LABEL   "";
    TYPE    SCOPE;
    MEMBERS
    {
        SOURCE1, chart_source_TV_VibrationPerc;
        SOURCE1, chart_source_TV_VibrationG;
    }
}
SOURCE  chart_source_TV_VibrationPerc
{
    LABEL   "";
    Y_AXIS  chart_TV_VibrationPerc_yaxis;
    MEMBERS
    {
        VALUE1, PZD_1013;
    }
    REFRESH_ACTIONS {
        DEFINITION {
            method_TV_Pos_refresh();
            method_TV_Vibration_refresh();
        }
    }
}
SOURCE  chart_source_TV_VibrationG
{
    LABEL   "";
    Y_AXIS  chart_TV_VibrationG_yaxis;
    MEMBERS
    {
        VALUE1, PZD_174;
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 *  GRAPHS
 * --------------------------------------------------------------------------------------------------------------------
 */
AXIS    xaxis_TV_Position
{
    LABEL            "Position";
    CONSTANT_UNIT    "%";
    MIN_VALUE    TV_MIN_POSITION;
    MAX_VALUE    TV_MAX_POSITION;
}

GRAPH graph_TV_TorquePos
{
    LABEL   "Torque over position|de|Drehmoment-Position"; /* [TXT_graph_TV_TorquePos]; */
    X_AXIS  xaxis_TV_Position;
    MEMBERS
    {
        WAVEFORM1, wave_TV_TORQUE;
    }
}

WAVEFORM  wave_TV_TORQUE
{
    LABEL   "Torque over position|de|Drehmoment-Position"; /* [TXT_graph_TV_TorquePos]; */
    Y_AXIS  chart_TV_TorqueNm_yaxis;
    TYPE    YT
    {
        X_INITIAL    TV_MIN_POSITION;
        X_INCREMENT  TV_POSITION_INCREMENT;
        NUMBER_OF_POINTS    TV_POSITION_VECLEN;
        Y_VALUES     
        {  
            arr_TV_Torque
        }
    }
}

GRAPH graph_TV_VibrationPos
{
    LABEL   "Vibration over position|de|Vibration-Position"; /* [TXT_graph_TV_VibrationPos]; */
    X_AXIS  xaxis_TV_Position;
    MEMBERS
    {
        WAVEFORM1, wave_TV_Vibration;
    }
}

WAVEFORM  wave_TV_Vibration
{
    LABEL   "Vibration over position|de|Vibration-Position"; /* [TXT_graph_TV_VibrationPos]; */
    Y_AXIS  chart_TV_VibrationPerc_yaxis;
    TYPE    YT
    {
        X_INITIAL    TV_MIN_POSITION;
        X_INCREMENT  TV_POSITION_INCREMENT;
        NUMBER_OF_POINTS    TV_POSITION_VECLEN;
        Y_VALUES     
        {  
            arr_TV_Vibration
        }
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 * METHODS
 * --------------------------------------------------------------------------------------------------------------------
 */
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_TorqueNm_init
 */ 
METHOD method_TV_TorqueNm_init
{
    LABEL       "method_TV_TorqueNm_init";
    DEFINITION
    {
        /* read PRM_1969 + PRM_2088 */
        ReadCommand(read_PRM_1969);
        ReadCommand(read_PRM_2088);

		/* Save refreshed data */
        save_values();
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_TorquePercent_refresh
 */ 
METHOD method_TV_TorquePercent_refresh
{
    LABEL       "method_TV_TorquePercent_refresh";
    DEFINITION
    {   
        float fval;
        float fval_abs;
		int iResult;
		
        /* read recent values from device */
        iResult= ReadCommand(read_PZD_545);  /* PZD_545 "Drehmoment AUF/ZU in % normiert auf 0 bis 1000"; Offset: -500, bezogen auf 127% */

        /* calculate signals */
		if (0 == iResult) {
            /* Signal TorquePercent (Berechnung wie in AC2) */
			fval= (PZD_545-500) * .254;
			if (fval < 0) { fval_abs= -fval; }  else { fval_abs= fval; }
		}
		else {
			PUT_MESSAGE ("Refresh of device data not possible|de|Lesen der Gertewerte nicht mglich");
			fval= var_TV_TorquePercent_Min;
		}
        var_TV_TorquePercent= fval;
        if (var_TV_TorquePercent_Min > fval) { var_TV_TorquePercent_Min= fval - .1*fval_abs; }
        if (var_TV_TorquePercent_Max < fval) { var_TV_TorquePercent_Max= fval + .1*fval_abs; }

		/* Save refreshed data */
        save_values();
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_TorqueNm_refresh
 */ 
METHOD method_TV_TorqueNm_refresh
{
    LABEL       "method_TV_TorqueNm_refresh";
    DEFINITION
    {   
        float fval;
		float fval_abs;
		int iResult;
        /* read recent values from device */
        iResult= ReadCommand(read_PZD_265);  /* PZD_265 "Fhrt ZU" */

        /* calculate signals */
		if (0 == iResult) {
			/* Signal TorqueNm */
			if (PZD_265 == 1) { fval= var_TV_TorquePercent/100.0 * PRM_2088; }
			else { fval= var_TV_TorquePercent/100.0 * PRM_1969; }
			if (fval < 0) { fval_abs= -fval; }  else { fval_abs= fval; }
		}
		else {
			PUT_MESSAGE ("Refresh of device data not possible|de|Lesen der Gertewerte nicht mglich");
			fval= var_TV_TorqueNm_Min;
		}
        var_TV_TorqueNm= fval;
        if (var_TV_TorqueNm_Min > fval) { var_TV_TorqueNm_Min= fval - .1*fval_abs; }
        if (var_TV_TorqueNm_Max < fval) { var_TV_TorqueNm_Max= fval + .1*fval_abs; }
        /* store in torque value array */
        arr_TV_Torque[var_TV_PosIndex]= var_TV_TorqueNm;

		/* Save refreshed data */
        save_values();
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_TorqueFtLb_refresh
 */ 
METHOD method_TV_TorqueFtLb_refresh
{
    LABEL       "method_TV_TorqueFtLb_refresh";
    DEFINITION
    {   
        float fval;
		float fval_abs;
        /* calculate signals */
        /* Signal TorqueFtLb */
        fval= K_Nm2FtLb * var_TV_TorqueNm;
		if (fval < 0) { fval_abs= -fval; }  else { fval_abs= fval; }		
        var_TV_TorqueFtLb= fval;
        if (var_TV_TorqueFtLb_Min > fval) { var_TV_TorqueFtLb_Min=  fval - .1*fval_abs; }
        if (var_TV_TorqueFtLb_Max < fval) { var_TV_TorqueFtLb_Max=  fval + .1*fval_abs; }

		/* Save refreshed data */
        save_values();
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_Pos_refresh
 */ 
METHOD method_TV_Pos_refresh
{
    LABEL       "method_TV_Pos_refresh";
    DEFINITION
    {   
		int iResult;
        /* read recent values from device */
        iResult= ReadCommand(read_PZD_2);    /* PZD_2 "Aktuelle Position [%]" */
        iResult += ReadCommand(read_PZD_683);  /* PZD_683 "Fahrbefehl-Feldbus" */

        /* calculate signals */
		if (0 == iResult) {
			/* Signal ActPos (per mille -> percent) */
			var_TV_ActPos= TV_POS_PZD2SCOPE * PZD_2;
			/* Signal SetPos (per mille -> percent) */
			var_TV_SetPos= TV_POS_PZD2SCOPE * PZD_683_position;
		}
		else {
			PUT_MESSAGE ("Refresh of device data not possible|de|Lesen der Gertewerte nicht mglich");
			/* Signal ActPos (per mille -> percent) */
			var_TV_ActPos= 0;
			/* Signal SetPos (per mille -> percent) */
			var_TV_SetPos= 0;
		}
		/* calculate position array index (PZD_2= {0..1000} -> array index= {0..100}) */
		var_TV_PosIndex= (var_TV_ActPos - TV_MIN_POSITION)/TV_POSITION_INCREMENT;
        if ( var_TV_PosIndex < TV_MIN_POSITION) { var_TV_PosIndex= TV_MIN_POSITION; }
        if ( var_TV_PosIndex > TV_MAX_POSITION) { var_TV_PosIndex= TV_MAX_POSITION; }

		/* Save refreshed data */
        save_values();
    }
}
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_TrvlStatus_refresh
 */ 
METHOD method_TV_TrvlStatus_refresh
{
    LABEL       "method_TV_TrvlStatus_refresh";
    DEFINITION
    {   
		int iResult;
        /* read recent values from device */
        iResult= ReadCommand(read_PZD_258);  /* PZD_258 "Endlage AUF" */
        iResult += ReadCommand(read_PZD_259);  /* PZD_259 "Endlage ZU" */
        iResult += ReadCommand(read_PZD_264);  /* PZD_264 "Fhrt AUF" */
        iResult += ReadCommand(read_PZD_265);  /* PZD_265 "Fhrt ZU" */
        iResult += ReadCommand(read_PZD_401);  /* PZD_401 "Sollposition erreicht" */

        /* calculate signals */
		if (0 == iResult) {
			/* Travel status */
			if (PZD_264 == 1)      var_TV_TrvlStatus= 1;
			else if (PZD_265 == 1) var_TV_TrvlStatus= 2;
			else if (PZD_258 == 1) var_TV_TrvlStatus= 3;
			else if (PZD_259 == 1) var_TV_TrvlStatus= 4;
			else if (PZD_401 == 1) var_TV_TrvlStatus= 5;
			else                   var_TV_TrvlStatus= 0;
		}
		else {
			PUT_MESSAGE ("Refresh of device data not possible|de|Lesen der Gertewerte nicht mglich");
			var_TV_TrvlStatus= 0;
		}

		/* Save refreshed data */
        save_values();
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining METHOD  method_TV_Vibration_refresh
 */ 
METHOD method_TV_Vibration_refresh
{
    LABEL       "method_TV_Vibration_refresh";
    DEFINITION
    {   
		int iResult;
        /* read recent values from device */
        iResult = ReadCommand(read_PZD_174);    /* PZD_174 "Beschleunigung Achse 1" */
        iResult += ReadCommand(read_PZD_1013);  /* PZD_1013 "Beschleunigung gesamt (XYZ)" */

		if (0 == iResult) {
			/* store in vibration value array */
			arr_TV_Vibration[var_TV_PosIndex]= PZD_1013;
		}
		else {
			PUT_MESSAGE ("Refresh of device data not possible|de|Lesen der Gertewerte nicht mglich");
			/* store in vibration value array */
			arr_TV_Vibration[var_TV_PosIndex]= 0;
		}

		/* Save refreshed data */
        save_values();
    }
}

/* --------------------------------------------------------------------------------------------------------------------
 * MENUS
 * --------------------------------------------------------------------------------------------------------------------
 */
/* --------------------------------------------------------------------------------------------------------------------
 *   Defining MENU  menu_TV_main -- root menu for asset management (trend view)
 */ 
MENU  menu_TV_main
{
    LABEL    "Trend chart|de|Trend-Anzeige";  /* [TXT_menu_TV_main]; */
    STYLE    WINDOW;
    ACCESS   OFFLINE;
    ITEMS
    {
        menu_TV_Torque
        , menu_TV_Position
        , menu_TV_Status
        , menu_TV_Vibration
    }   
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining MENU  menu_TV_Torque -- menu for trend chart torque
 */ 
MENU  menu_TV_Torque
{
    LABEL    "Torque|de|Drehmoment";  /* [TXT_menu_TV_torque]; */
    STYLE    PAGE;
    ACCESS   OFFLINE;
    ITEMS
    {
        chart_TV_Torque
        , var_TV_TorqueFtLb
        , var_TV_TorqueNm
        , var_TV_TorquePercent
        , graph_TV_TorquePos
    }   
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining MENU  menu_TV_Vibration -- menu for trend chart drive vibration
 */ 
MENU  menu_TV_Vibration
{
    LABEL    "Vibration";  /* [TXT_menu_TV_Vibration]; */
    STYLE    PAGE;
    ACCESS   OFFLINE;
    ITEMS
    {
        chart_TV_Vibration
        , PZD_1013
        , PZD_174
        , graph_TV_VibrationPos
    }   
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining MENU  menu_TV_Position -- menu for trend chart position
 */ 
MENU  menu_TV_Position
{
    LABEL    "Position";  /* [TXT_menu_TV_Position]; */
    STYLE    PAGE;
    ACCESS   OFFLINE;
    ITEMS
    {
        chart_TV_Position
        , var_TV_SetPos
        , var_TV_ActPos
    }   
}

/* --------------------------------------------------------------------------------------------------------------------
 *   Defining MENU  menu_TV_Status -- menu for trend chart drive status
 */ 
MENU  menu_TV_Status
{
    LABEL    "Status";  /* [TXT_menu_TV_Status]; */
    STYLE    PAGE;
    ACCESS   OFFLINE;
    ITEMS
    {
        chart_TV_Status
        , var_TV_TrvlStatus
    }   
}

/* ---------------------------------------------------------------------------------------------------------------------
 *   End of file 
 * ---------------------------------------------------------------------------------------------------------------------
 */